@@ -17,6 +17,7 @@
 import logging
 import traceback
 from collections import defaultdict
+import inspect
 
 import beets
 from beets import mediafile
@@ -30,6 +31,14 @@ LASTFM_KEY = '2dc3914abf35f0d9c92d97d8f8e42b43'
 log = logging.getLogger('beets')
 
 
+class PluginConflictException(Exception):
+    """Indicates that the services provided by one plugin conflict with
+    those of another.
+
+    For example two plugins may define different types for flexible fields.
+    """
+
+
 # Managing the plugins themselves.
 
 class BeetsPlugin(object):
@@ -40,7 +49,6 @@ class BeetsPlugin(object):
     def __init__(self, name=None):
         """Perform one-time plugin setup.
         """
-        _add_media_fields(self.item_fields())
         self.import_stages = []
         self.name = name or self.__module__.split('.')[-1]
         self.config = beets.config[self.name]
@@ -56,10 +64,9 @@ class BeetsPlugin(object):
         commands that should be added to beets' CLI.
         """
         return ()
-    
+
     def queries(self):
-        """Should return a dict mapping prefixes to PluginQuery
-        subclasses.
+        """Should return a dict mapping prefixes to Query subclasses.
         """
         return {}
 
@@ -87,20 +94,6 @@ class BeetsPlugin(object):
         """
         return ()
 
-    def configure(self, config):
-        """This method is called with the ConfigParser object after
-        the CLI starts up.
-        """
-        pass
-
-    def item_fields(self):
-        """Returns field descriptors to be added to the MediaFile class,
-        in the form of a dictionary whose keys are field names and whose
-        values are descriptor (e.g., MediaField) instances. The Library
-        database schema is not (currently) extended.
-        """
-        return {}
-
     def album_for_id(self, album_id):
         """Return an AlbumInfo object or None if no matching release was
         found.
@@ -113,6 +106,20 @@ class BeetsPlugin(object):
         """
         return None
 
+    def add_media_field(self, name, descriptor):
+        """Add a field that is synchronized between media files and items.
+
+        When a media field is added ``item.write()`` will set the name
+        property of the item's MediaFile to ``item[name]`` and save the
+        changes. Similarly ``item.read()`` will set ``item[name]`` to
+        the value of the name property of the media file.
+
+        ``descriptor`` must be an instance of ``mediafile.MediaField``.
+        """
+        # Defer impor to prevent circular dependency
+        from beets import library
+        mediafile.MediaFile.add_field(name, descriptor)
+        library.Item._media_fields.add(name)
 
     listeners = None
 
@@ -137,7 +144,7 @@ class BeetsPlugin(object):
 
             >>> @MyPlugin.listen("imported")
             >>> def importListener(**kwargs):
-            >>>     pass
+            ...     pass
         """
         def helper(func):
             if cls.listeners is None:
@@ -177,7 +184,10 @@ class BeetsPlugin(object):
             return func
         return helper
 
-_classes = []
+
+_classes = set()
+
+
 def load_plugins(names=()):
     """Imports the modules for a sequence of plugin names. Each name
     must be the name of a Python module under the "beetsplug" namespace
@@ -192,20 +202,23 @@ def load_plugins(names=()):
             except ImportError as exc:
                 # Again, this is hacky:
                 if exc.args[0].endswith(' ' + name):
-                    log.warn('** plugin %s not found' % name)
+                    log.warn(u'** plugin {0} not found'.format(name))
                 else:
                     raise
             else:
                 for obj in getattr(namespace, name).__dict__.values():
                     if isinstance(obj, type) and issubclass(obj, BeetsPlugin) \
-                            and obj != BeetsPlugin:
-                        _classes.append(obj)
+                            and obj != BeetsPlugin and obj not in _classes:
+                        _classes.add(obj)
 
         except:
-            log.warn('** error loading plugin %s' % name)
+            log.warn(u'** error loading plugin {0}'.format(name))
             log.warn(traceback.format_exc())
 
+
 _instances = {}
+
+
 def find_plugins():
     """Returns a list of BeetsPlugin subclass instances from all
     currently loaded beets plugins. Loads the default plugin set
@@ -231,15 +244,33 @@ def commands():
         out += plugin.commands()
     return out
 
+
 def queries():
-    """Returns a dict mapping prefix strings to beet.library.PluginQuery
-    subclasses all loaded plugins.
+    """Returns a dict mapping prefix strings to Query subclasses all loaded
+    plugins.
     """
     out = {}
     for plugin in find_plugins():
         out.update(plugin.queries())
     return out
 
+
+def types(model_cls):
+    # Gives us `item_types` and `album_types`
+    attr_name = '{0}_types'.format(model_cls.__name__.lower())
+    types = {}
+    for plugin in find_plugins():
+        plugin_types = getattr(plugin, attr_name, {})
+        for field in plugin_types:
+            if field in types:
+                raise PluginConflictException(
+                    u'Plugin {0} defines flexible field {1} '
+                    'which has already been defined.'
+                    .format(plugin.name,))
+        types.update(plugin_types)
+    return types
+
+
 def track_distance(item, info):
     """Gets the track distance calculated by all loaded plugins.
     Returns a Distance object.
@@ -250,6 +281,7 @@ def track_distance(item, info):
         dist.update(plugin.track_distance(item, info))
     return dist
 
+
 def album_distance(items, album_info, mapping):
     """Returns the album distance calculated by plugins."""
     from beets.autotag.hooks import Distance
@@ -258,6 +290,7 @@ def album_distance(items, album_info, mapping):
         dist.update(plugin.album_distance(items, album_info, mapping))
     return dist
 
+
 def candidates(items, artist, album, va_likely):
     """Gets MusicBrainz candidates for an album from each plugin.
     """
@@ -266,6 +299,7 @@ def candidates(items, artist, album, va_likely):
         out.extend(plugin.candidates(items, artist, album, va_likely))
     return out
 
+
 def item_candidates(item, artist, title):
     """Gets MusicBrainz candidates for an item from the plugins.
     """
@@ -274,6 +308,7 @@ def item_candidates(item, artist, title):
         out.extend(plugin.item_candidates(item, artist, title))
     return out
 
+
 def album_for_id(album_id):
     """Get AlbumInfo objects for a given ID string.
     """
@@ -284,6 +319,7 @@ def album_for_id(album_id):
             out.append(res)
     return out
 
+
 def track_for_id(track_id):
     """Get TrackInfo objects for a given ID string.
     """
@@ -294,10 +330,6 @@ def track_for_id(track_id):
             out.append(res)
     return out
 
-def configure(config):
-    """Sends the configuration object to each plugin."""
-    for plugin in find_plugins():
-        plugin.configure(config)
 
 def template_funcs():
     """Get all the template functions declared by plugins as a
@@ -309,33 +341,6 @@ def template_funcs():
             funcs.update(plugin.template_funcs)
     return funcs
 
-def template_values(item):
-    """Get all the template values computed for a given Item by
-    registered field computations.
-    """
-    values = {}
-    for plugin in find_plugins():
-        if plugin.template_fields:
-            for name, func in plugin.template_fields.iteritems():
-                values[name] = unicode(func(item))
-    return values
-
-def album_template_values(album):
-    """Get the plugin-defined template values for an Album.
-    """
-    values = {}
-    for plugin in find_plugins():
-        if plugin.album_template_fields:
-            for name, func in plugin.album_template_fields.iteritems():
-                values[name] = unicode(func(album))
-    return values
-
-def _add_media_fields(fields):
-    """Adds a {name: descriptor} dictionary of fields to the MediaFile
-    class. Called during the plugin initialization.
-    """
-    for key, value in fields.iteritems():
-        setattr(mediafile.MediaFile, key, value)
 
 def import_stages():
     """Get a list of import stage functions defined by plugins."""
@@ -346,6 +351,29 @@ def import_stages():
     return stages
 
 
+# New-style (lazy) plugin-provided fields.
+
+def item_field_getters():
+    """Get a dictionary mapping field names to unary functions that
+    compute the field's value.
+    """
+    funcs = {}
+    for plugin in find_plugins():
+        if plugin.template_fields:
+            funcs.update(plugin.template_fields)
+    return funcs
+
+
+def album_field_getters():
+    """As above, for album fields.
+    """
+    funcs = {}
+    for plugin in find_plugins():
+        if plugin.album_template_fields:
+            funcs.update(plugin.album_template_fields)
+    return funcs
+
+
 # Event dispatch.
 
 def event_handlers():
@@ -359,15 +387,17 @@ def event_handlers():
                 all_handlers[event] += handlers
     return all_handlers
 
+
 def send(event, **arguments):
     """Sends an event to all assigned event listeners. Event is the
     name of  the event to send, all other named arguments go to the
     event handler(s).
 
-    Returns the number of handlers called.
+    Returns a list of return values from the handlers.
     """
-    log.debug('Sending event: %s' % event)
-    handlers = event_handlers()[event]
-    for handler in handlers:
-        handler(**arguments)
-    return len(handlers)
+    log.debug(u'Sending event: {0}'.format(event))
+    for handler in event_handlers()[event]:
+        # Don't break legacy plugins if we want to pass more arguments
+        argspec = inspect.getargspec(handler).args
+        args = dict((k, v) for k, v in arguments.items() if k in argspec)
+        handler(**args)
