@@ -30,7 +30,9 @@ from beets.plugins import BeetsPlugin
 import beets.ui
 from beets import vfs
 from beets.util import bluelet
-from beets.library import ITEM_KEYS_WRITABLE
+from beets.library import Item
+from beets import dbcore
+from beets.mediafile import MediaFile
 
 PROTOCOL_VERSION = '0.13.0'
 BUFSIZE = 1024
@@ -66,6 +68,7 @@ SAFE_COMMANDS = (
     u'close', u'commands', u'notcommands', u'password', u'ping',
 )
 
+ITEM_KEYS_WRITABLE = set(MediaFile.fields()).intersection(Item._fields.keys())
 
 # Loggers.
 log = logging.getLogger('beets.bpd')
@@ -73,7 +76,8 @@ global_log = logging.getLogger('beets')
 
 
 # Gstreamer import error.
-class NoGstreamerError(Exception): pass
+class NoGstreamerError(Exception):
+    pass
 
 
 # Error-handling, exceptions, parameter parsing.
@@ -89,32 +93,39 @@ class BPDError(Exception):
         self.index = index
 
     template = Template(u'$resp [$code@$index] {$cmd_name} $message')
+
     def response(self):
         """Returns a string to be used as the response code for the
         erring command.
         """
-        return self.template.substitute({'resp':     RESP_ERR,
-                                         'code':     self.code,
-                                         'index':    self.index,
-                                         'cmd_name': self.cmd_name,
-                                         'message':  self.message
-                                       })
+        return self.template.substitute({
+            'resp':     RESP_ERR,
+            'code':     self.code,
+            'index':    self.index,
+            'cmd_name': self.cmd_name,
+            'message':  self.message,
+        })
+
 
 def make_bpd_error(s_code, s_message):
     """Create a BPDError subclass for a static code and message.
     """
+
     class NewBPDError(BPDError):
         code = s_code
         message = s_message
         cmd_name = ''
         index = 0
-        def __init__(self): pass
+
+        def __init__(self):
+            pass
     return NewBPDError
 
 ArgumentTypeError = make_bpd_error(ERROR_ARG, 'invalid type for argument')
 ArgumentIndexError = make_bpd_error(ERROR_ARG, 'argument out of range')
 ArgumentNotFoundError = make_bpd_error(ERROR_NO_EXIST, 'argument not found')
 
+
 def cast_arg(t, val):
     """Attempts to call t on val, raising a ArgumentTypeError
     on ValueError.
@@ -130,14 +141,15 @@ def cast_arg(t, val):
         except ValueError:
             raise ArgumentTypeError()
 
+
 class BPDClose(Exception):
     """Raised by a command invocation to indicate that the connection
     should be closed.
     """
 
-
 # Generic server infrastructure, implementing the basic protocol.
 
+
 class BaseServer(object):
     """A MPD-compatible music player server.
 
@@ -208,10 +220,10 @@ class BaseServer(object):
         If there is only one song in the playlist it returns 0.
         """
         if len(self.playlist) < 2:
-            return len(self.playlist)-1
-        new_index = self.random_obj.randint(0, len(self.playlist)-1)
+            return len(self.playlist) - 1
+        new_index = self.random_obj.randint(0, len(self.playlist) - 1)
         while new_index == self.current_index:
-            new_index = self.random_obj.randint(0, len(self.playlist)-1)
+            new_index = self.random_obj.randint(0, len(self.playlist) - 1)
         return new_index
 
     def _succ_idx(self):
@@ -223,7 +235,7 @@ class BaseServer(object):
             return self.current_index
         if self.random:
             return self._random_idx()
-        return self.current_index+1
+        return self.current_index + 1
 
     def _prev_idx(self):
         """Returns the index for the previous song to play.
@@ -234,7 +246,7 @@ class BaseServer(object):
             return self.current_index
         if self.random:
             return self._random_idx()
-        return self.current_index-1
+        return self.current_index - 1
 
     def cmd_ping(self, conn):
         """Succeeds."""
@@ -290,13 +302,14 @@ class BaseServer(object):
         Gives a list of response-lines for: volume, repeat, random,
         playlist, playlistlength, and xfade.
         """
-        yield (u'volume: ' + unicode(self.volume),
-               u'repeat: ' + unicode(int(self.repeat)),
-               u'random: ' + unicode(int(self.random)),
-               u'playlist: ' + unicode(self.playlist_version),
-               u'playlistlength: ' + unicode(len(self.playlist)),
-               u'xfade: ' + unicode(self.crossfade),
-              )
+        yield (
+            u'volume: ' + unicode(self.volume),
+            u'repeat: ' + unicode(int(self.repeat)),
+            u'random: ' + unicode(int(self.random)),
+            u'playlist: ' + unicode(self.playlist_version),
+            u'playlistlength: ' + unicode(len(self.playlist)),
+            u'xfade: ' + unicode(self.crossfade),
+        )
 
         if self.current_index == -1:
             state = u'stop'
@@ -306,7 +319,7 @@ class BaseServer(object):
             state = u'play'
         yield u'state: ' + state
 
-        if self.current_index != -1: # i.e., paused or playing
+        if self.current_index != -1:  # i.e., paused or playing
             current_id = self._item_id(self.playlist[self.current_index])
             yield u'song: ' + unicode(self.current_index)
             yield u'songid: ' + unicode(current_id)
@@ -357,9 +370,9 @@ class BaseServer(object):
             raise ArgumentIndexError()
         self.playlist_version += 1
 
-        if self.current_index == index: # Deleted playing song.
+        if self.current_index == index:  # Deleted playing song.
             self.cmd_stop(conn)
-        elif index < self.current_index: # Deleted before playing.
+        elif index < self.current_index:  # Deleted before playing.
             # Shift playing index down.
             self.current_index -= 1
 
@@ -434,6 +447,7 @@ class BaseServer(object):
             except IndexError:
                 raise ArgumentIndexError()
             yield self._item_info(track)
+
     def cmd_playlistid(self, conn, track_id=-1):
         return self.cmd_playlistinfo(conn, self._id_to_index(track_id))
 
@@ -458,7 +472,7 @@ class BaseServer(object):
     def cmd_currentsong(self, conn):
         """Sends information about the currently-playing song.
         """
-        if self.current_index != -1: # -1 means stopped.
+        if self.current_index != -1:  # -1 means stopped.
             track = self.playlist[self.current_index]
             yield self._item_info(track)
 
@@ -482,7 +496,7 @@ class BaseServer(object):
     def cmd_pause(self, conn, state=None):
         """Set the pause state playback."""
         if state is None:
-            self.paused = not self.paused # Toggle.
+            self.paused = not self.paused  # Toggle.
         else:
             self.paused = cast_arg('intbool', state)
 
@@ -493,14 +507,14 @@ class BaseServer(object):
         if index < -1 or index > len(self.playlist):
             raise ArgumentIndexError()
 
-        if index == -1: # No index specified: start where we are.
-            if not self.playlist: # Empty playlist: stop immediately.
+        if index == -1:  # No index specified: start where we are.
+            if not self.playlist:  # Empty playlist: stop immediately.
                 return self.cmd_stop(conn)
-            if self.current_index == -1: # No current song.
-                self.current_index = 0 # Start at the beginning.
+            if self.current_index == -1:  # No current song.
+                self.current_index = 0  # Start at the beginning.
             # If we have a current song, just stay there.
 
-        else: # Start with the specified index.
+        else:  # Start with the specified index.
             self.current_index = index
 
         self.paused = False
@@ -524,6 +538,7 @@ class BaseServer(object):
         if index < 0 or index >= len(self.playlist):
             raise ArgumentIndexError()
         self.current_index = index
+
     def cmd_seekid(self, conn, track_id, pos):
         index = self._id_to_index(track_id)
         return self.cmd_seek(conn, index, pos)
@@ -534,6 +549,7 @@ class BaseServer(object):
         heap = hpy().heap()
         print(heap)
 
+
 class Connection(object):
     """A connection between a client and the server. Handles input and
     output from and to the client.
@@ -554,7 +570,7 @@ class Connection(object):
         if isinstance(lines, basestring):
             lines = [lines]
         out = NEWLINE.join(lines) + NEWLINE
-        log.debug(out[:-1]) # Don't log trailing newline.
+        log.debug(out[:-1])  # Don't log trailing newline.
         if isinstance(out, unicode):
             out = out.encode('utf8')
         return self.sock.sendall(out)
@@ -577,9 +593,9 @@ class Connection(object):
         """
         yield self.send(HELLO)
 
-        clist = None # Initially, no command list is being constructed.
+        clist = None  # Initially, no command list is being constructed.
         while True:
-            line = yield self.sock.readline() 
+            line = yield self.sock.readline()
             if not line:
                 break
             line = line.strip()
@@ -591,7 +607,7 @@ class Connection(object):
                 # Command list already opened.
                 if line == CLIST_END:
                     yield bluelet.call(self.do_command(clist))
-                    clist = None # Clear the command list.
+                    clist = None  # Clear the command list.
                 else:
                     clist.append(Command(line))
 
@@ -616,6 +632,7 @@ class Connection(object):
             return cls(server, sock).run()
         return _handle
 
+
 class Command(object):
     """A command issued by the client for processing by the server.
     """
@@ -688,6 +705,7 @@ class CommandList(list):
     server. May be verbose, in which case the response is delimited, or
     not. Should be a list of `Command` objects.
     """
+
     def __init__(self, sequence=None, verbose=False):
         """Create a new `CommandList` from the given sequence of
         `Command`s. If `verbose`, this is a verbose command list.
@@ -705,7 +723,7 @@ class CommandList(list):
                 yield bluelet.call(command.run(conn))
             except BPDError as e:
                 # If the command failed, stop executing.
-                e.index = i # Give the error the correct index.
+                e.index = i  # Give the error the correct index.
                 raise e
 
             # Otherwise, possibly send the output delimeter if we're in a
@@ -714,7 +732,6 @@ class CommandList(list):
                 yield conn.send(RESP_CLIST_VERBOSE)
 
 
-
 # A subclass of the basic, protocol-handling server that actually plays
 # music.
 
@@ -747,17 +764,17 @@ class Server(BaseServer):
         """
         self.cmd_next(None)
 
-
     # Metadata helper functions.
 
     def _item_info(self, item):
-        info_lines = [u'file: ' + item.destination(fragment=True),
-                      u'Time: ' + unicode(int(item.length)),
-                      u'Title: ' + item.title,
-                      u'Artist: ' + item.artist,
-                      u'Album: ' + item.album,
-                      u'Genre: ' + item.genre,
-                     ]
+        info_lines = [
+            u'file: ' + item.destination(fragment=True),
+            u'Time: ' + unicode(int(item.length)),
+            u'Title: ' + item.title,
+            u'Artist: ' + item.artist,
+            u'Album: ' + item.album,
+            u'Genre: ' + item.genre,
+        ]
 
         track = unicode(item.track)
         if item.tracktotal:
@@ -780,7 +797,6 @@ class Server(BaseServer):
     def _item_id(self, item):
         return item.id
 
-
     # Database updating.
 
     def cmd_update(self, conn, path=u'/'):
@@ -793,7 +809,6 @@ class Server(BaseServer):
         print('... done.')
         self.updated_time = time.time()
 
-
     # Path (directory tree) browsing.
 
     def _resolve_path(self, path):
@@ -858,20 +873,22 @@ class Server(BaseServer):
             for name, itemid in sorted(node.files.iteritems()):
                 newpath = self._path_join(basepath, name)
                 # "yield from"
-                for v in self._listall(newpath, itemid, info): yield v
+                for v in self._listall(newpath, itemid, info):
+                    yield v
             for name, subdir in sorted(node.dirs.iteritems()):
                 newpath = self._path_join(basepath, name)
                 yield u'directory: ' + newpath
-                for v in self._listall(newpath, subdir, info): yield v
+                for v in self._listall(newpath, subdir, info):
+                    yield v
 
     def cmd_listall(self, conn, path=u"/"):
         """Send the paths all items in the directory, recursively."""
         return self._listall(path, self._resolve_path(path), False)
+
     def cmd_listallinfo(self, conn, path=u"/"):
         """Send info on all the items in the directory, recursively."""
         return self._listall(path, self._resolve_path(path), True)
 
-
     # Playlist manipulation.
 
     def _all_items(self, node):
@@ -885,9 +902,11 @@ class Server(BaseServer):
             # Recurse into a directory.
             for name, itemid in sorted(node.files.iteritems()):
                 # "yield from"
-                for v in self._all_items(itemid): yield v
+                for v in self._all_items(itemid):
+                    yield v
             for name, subdir in sorted(node.dirs.iteritems()):
-                for v in self._all_items(subdir): yield v
+                for v in self._all_items(subdir):
+                    yield v
 
     def _add(self, path, send_id=False):
         """Adds a track or directory to the playlist, specified by the
@@ -909,7 +928,6 @@ class Server(BaseServer):
         """Same as `cmd_add` but sends an id back to the client."""
         return self._add(path, True)
 
-
     # Server info.
 
     def cmd_status(self, conn):
@@ -918,7 +936,7 @@ class Server(BaseServer):
         if self.current_index > -1:
             item = self.playlist[self.current_index]
 
-            yield u'bitrate: ' + unicode(item.bitrate/1000)
+            yield u'bitrate: ' + unicode(item.bitrate / 1000)
             # Missing 'audio'.
 
             (pos, total) = self.player.time()
@@ -926,7 +944,6 @@ class Server(BaseServer):
 
         # Also missing 'updating_db'.
 
-
     def cmd_stats(self, conn):
         """Sends some statistics about the library."""
         with self.lib.transaction() as tx:
@@ -937,15 +954,15 @@ class Server(BaseServer):
                         'FROM items'
             artists, albums, songs, totaltime = tx.query(statement)[0]
 
-        yield (u'artists: ' + unicode(artists),
-               u'albums: ' + unicode(albums),
-               u'songs: ' + unicode(songs),
-               u'uptime: ' + unicode(int(time.time() - self.startup_time)),
-               u'playtime: ' + u'0', # Missing.
-               u'db_playtime: ' + unicode(int(totaltime)),
-               u'db_update: ' + unicode(int(self.updated_time)),
-              )
-
+        yield (
+            u'artists: ' + unicode(artists),
+            u'albums: ' + unicode(albums),
+            u'songs: ' + unicode(songs),
+            u'uptime: ' + unicode(int(time.time() - self.startup_time)),
+            u'playtime: ' + u'0',  # Missing.
+            u'db_playtime: ' + unicode(int(totaltime)),
+            u'db_update: ' + unicode(int(self.updated_time)),
+        )
 
     # Searching.
 
@@ -962,7 +979,7 @@ class Server(BaseServer):
         u'Composer':        u'composer',
         # Performer?
         u'Disc':            u'disc',
-        u'filename':        u'path', # Suspect.
+        u'filename':        u'path',  # Suspect.
     }
 
     def cmd_tagtypes(self, conn):
@@ -990,34 +1007,36 @@ class Server(BaseServer):
         pairs specified. The any_query_type is used for queries of
         type "any"; if None, then an error is thrown.
         """
-        if kv: # At least one key-value pair.
+        if kv:  # At least one key-value pair.
             queries = []
             # Iterate pairwise over the arguments.
             it = iter(kv)
             for tag, value in zip(it, it):
                 if tag.lower() == u'any':
                     if any_query_type:
-                        queries.append(any_query_type(value, ITEM_KEYS_WRITABLE, query_type))
+                        queries.append(any_query_type(value,
+                                                      ITEM_KEYS_WRITABLE,
+                                                      query_type))
                     else:
                         raise BPDError(ERROR_UNKNOWN, u'no such tagtype')
                 else:
                     _, key = self._tagtype_lookup(tag)
                     queries.append(query_type(key, value))
-            return beets.library.AndQuery(queries)
-        else: # No key-value pairs.
-            return beets.library.TrueQuery()
+            return dbcore.query.AndQuery(queries)
+        else:  # No key-value pairs.
+            return dbcore.query.TrueQuery()
 
     def cmd_search(self, conn, *kv):
         """Perform a substring match for items."""
-        query = self._metadata_query(beets.library.SubstringQuery,
-                                     beets.library.AnyFieldQuery,
+        query = self._metadata_query(dbcore.query.SubstringQuery,
+                                     dbcore.query.AnyFieldQuery,
                                      kv)
         for item in self.lib.items(query):
             yield self._item_info(item)
 
     def cmd_find(self, conn, *kv):
         """Perform an exact match for items."""
-        query = self._metadata_query(beets.library.MatchQuery,
+        query = self._metadata_query(dbcore.query.MatchQuery,
                                      None,
                                      kv)
         for item in self.lib.items(query):
@@ -1028,7 +1047,7 @@ class Server(BaseServer):
         filtered by matching match_tag to match_term.
         """
         show_tag_canon, show_key = self._tagtype_lookup(show_tag)
-        query = self._metadata_query(beets.library.MatchQuery, None, kv)
+        query = self._metadata_query(dbcore.query.MatchQuery, None, kv)
 
         clause, subvals = query.clause()
         statement = 'SELECT DISTINCT ' + show_key + \
@@ -1047,22 +1066,22 @@ class Server(BaseServer):
         _, key = self._tagtype_lookup(tag)
         songs = 0
         playtime = 0.0
-        for item in self.lib.items(beets.library.MatchQuery(key, value)):
+        for item in self.lib.items(dbcore.query.MatchQuery(key, value)):
             songs += 1
             playtime += item.length
         yield u'songs: ' + unicode(songs)
         yield u'playtime: ' + unicode(int(playtime))
 
-
     # "Outputs." Just a dummy implementation because we don't control
     # any outputs.
 
     def cmd_outputs(self, conn):
         """List the available outputs."""
-        yield (u'outputid: 0',
-               u'outputname: gstreamer',
-               u'outputenabled: 1',
-              )
+        yield (
+            u'outputid: 0',
+            u'outputname: gstreamer',
+            u'outputenabled: 1',
+        )
 
     def cmd_enableoutput(self, conn, output_id):
         output_id = cast_arg(int, output_id)
@@ -1076,7 +1095,6 @@ class Server(BaseServer):
         else:
             raise ArgumentIndexError()
 
-
     # Playback control. The functions below hook into the
     # half-implementations provided by the base class. Together, they're
     # enough to implement all normal playback functionality.
@@ -1086,7 +1104,7 @@ class Server(BaseServer):
         was_paused = self.paused
         super(Server, self).cmd_play(conn, index)
 
-        if self.current_index > -1: # Not stopped.
+        if self.current_index > -1:  # Not stopped.
             if was_paused and not new_index:
                 # Just unpause.
                 self.player.play()
@@ -1111,13 +1129,12 @@ class Server(BaseServer):
         super(Server, self).cmd_seek(conn, index, pos)
         self.player.seek(pos)
 
-
     # Volume control.
 
     def cmd_setvol(self, conn, vol):
         vol = cast_arg(int, vol)
         super(Server, self).cmd_setvol(conn, vol)
-        self.player.volume = float(vol)/100
+        self.player.volume = float(vol) / 100
 
 
 # Beets plugin hooks.
@@ -1143,15 +1160,18 @@ class BPDPlugin(BeetsPlugin):
         try:
             Server(lib, host, port, password).run()
         except NoGstreamerError:
-            global_log.error('Gstreamer Python bindings not found.')
-            global_log.error('Install "python-gst0.10", "py27-gst-python", '
-                             'or similar package to use BPD.')
+            global_log.error(u'Gstreamer Python bindings not found.')
+            global_log.error(u'Install "python-gst0.10", "py27-gst-python", '
+                             u'or similar package to use BPD.')
 
     def commands(self):
-        cmd = beets.ui.Subcommand('bpd',
-            help='run an MPD-compatible music player server')
-        cmd.parser.add_option('-d', '--debug', action='store_true',
-            help='dump all MPD traffic to stdout')
+        cmd = beets.ui.Subcommand(
+            'bpd', help='run an MPD-compatible music player server'
+        )
+        cmd.parser.add_option(
+            '-d', '--debug', action='store_true',
+            help='dump all MPD traffic to stdout'
+        )
 
         def func(lib, opts, args):
             host = args.pop(0) if args else self.config['host'].get(unicode)
