@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2014, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -13,6 +13,7 @@
 # included in all copies or substantial portions of the Software.
 
 """Allows beets to embed album art into file metadata."""
+import os.path
 import logging
 import imghdr
 
@@ -26,38 +27,6 @@ from beets import config
 
 log = logging.getLogger('beets')
 
-def _embed(path, items, maxwidth=0):
-    """Embed an image file, located at `path`, into each item.
-    """
-    if maxwidth:
-        path = ArtResizer.shared.resize(maxwidth, syspath(path))
-
-    data = open(syspath(path), 'rb').read()
-    kindstr = imghdr.what(None, data)
-    if kindstr is None:
-        log.error(u'Could not embed art of unkown type: {0}'.format(
-            displayable_path(path)
-        ))
-        return
-    elif kindstr not in ('jpeg', 'png'):
-        log.error(u'Image type {0} is not allowed as cover art: {1}'.format(
-            kindstr, displayable_path(path)
-        ))
-        return
-
-    # Add art to each file.
-    log.debug('Embedding album art.')
-
-    for item in items:
-        try:
-            f = mediafile.MediaFile(syspath(item.path))
-        except mediafile.UnreadableFileError as exc:
-            log.warn('Could not embed art in {0}: {1}'.format(
-                displayable_path(item.path), exc
-            ))
-            continue
-        f.art = data
-        f.save()
 
 class EmbedCoverArtPlugin(BeetsPlugin):
     """Allows albumart to be embedded into the actual files.
@@ -71,21 +40,28 @@ class EmbedCoverArtPlugin(BeetsPlugin):
         if self.config['maxwidth'].get(int) and \
                 not ArtResizer.shared.local:
             self.config['maxwidth'] = 0
-            log.warn("embedart: ImageMagick or PIL not found; "
-                     "'maxwidth' option ignored")
+            log.warn(u"embedart: ImageMagick or PIL not found; "
+                     u"'maxwidth' option ignored")
 
     def commands(self):
         # Embed command.
-        embed_cmd = ui.Subcommand('embedart',
-            help='embed image files into file metadata')
-        embed_cmd.parser.add_option('-f', '--file', metavar='PATH',
-            help='the image file to embed')
+        embed_cmd = ui.Subcommand(
+            'embedart', help='embed image files into file metadata'
+        )
+        embed_cmd.parser.add_option(
+            '-f', '--file', metavar='PATH', help='the image file to embed'
+        )
+        maxwidth = config['embedart']['maxwidth'].get(int)
+
         def embed_func(lib, opts, args):
             if opts.file:
                 imagepath = normpath(opts.file)
-                embed(lib, imagepath, decargs(args))
+                for item in lib.items(decargs(args)):
+                    embed_item(item, imagepath, maxwidth)
             else:
-                embed_current(lib, decargs(args))
+                for album in lib.albums(decargs(args)):
+                    embed_album(album, maxwidth)
+
         embed_cmd.func = embed_func
 
         # Extract command.
@@ -93,6 +69,7 @@ class EmbedCoverArtPlugin(BeetsPlugin):
                                     help='extract an image from file metadata')
         extract_cmd.parser.add_option('-o', dest='outpath',
                                       help='image output file')
+
         def extract_func(lib, opts, args):
             outpath = normpath(opts.outpath or 'cover')
             extract(lib, outpath, decargs(args))
@@ -101,50 +78,74 @@ class EmbedCoverArtPlugin(BeetsPlugin):
         # Clear command.
         clear_cmd = ui.Subcommand('clearart',
                                   help='remove images from file metadata')
+
         def clear_func(lib, opts, args):
             clear(lib, decargs(args))
         clear_cmd.func = clear_func
 
         return [embed_cmd, extract_cmd, clear_cmd]
 
-# "embedart" command with --file argument.
-def embed(lib, imagepath, query):
-    albums = lib.albums(query)
-    for i_album in albums:
-        album = i_album
-        break
-    else:
-        log.error('No album matches query.')
+
+@EmbedCoverArtPlugin.listen('album_imported')
+def album_imported(lib, album):
+    """Automatically embed art into imported albums.
+    """
+    if album.artpath and config['embedart']['auto']:
+        embed_album(album, config['embedart']['maxwidth'].get(int))
+
+
+def embed_item(item, imagepath, maxwidth=None, itempath=None):
+    """Embed an image into the item's media file.
+    """
+    try:
+        item['images'] = [_mediafile_image(imagepath, maxwidth)]
+        item.try_write(itempath)
+    except IOError as exc:
+        log.error(u'embedart: could not read image file: {0}'.format(exc))
+    finally:
+        # We don't want to store the image in the database
+        del item['images']
+
+
+def embed_album(album, maxwidth=None):
+    """Embed album art into all of the album's items.
+    """
+    imagepath = album.artpath
+    if not imagepath:
+        log.info(u'No album art present: {0} - {1}'.
+                 format(album.albumartist, album.album))
+        return
+    if not os.path.isfile(imagepath):
+        log.error(u'Album art not found at {0}'
+                  .format(imagepath))
         return
 
-    log.info(u'Embedding album art into {0.albumartist} - {0.album}.'.format(
-        album
-    ))
-    _embed(imagepath, album.items(),
-           config['embedart']['maxwidth'].get(int))
+    log.info(u'Embedding album art into {0.albumartist} - {0.album}.'
+             .format(album))
 
-# "embedart" command without explicit file.
-def embed_current(lib, query):
-    albums = lib.albums(query)
-    for album in albums:
-        if not album.artpath:
-            log.info(u'No album art present: {0} - {1}'.
-                     format(album.albumartist, album.album))
-            continue
+    for item in album.items():
+        embed_item(item, imagepath, maxwidth)
 
-        log.info(u'Embedding album art into {0} - {1}'.
-                 format(album.albumartist, album.album))
-        _embed(album.artpath, album.items(),
-               config['embedart']['maxwidth'].get(int))
 
-# "extractart" command.
+def _mediafile_image(image_path, maxwidth=None):
+    """Return a `mediafile.Image` object for the path.
+
+    If maxwidth is set the image is resized if necessary.
+    """
+    if maxwidth:
+        image_path = ArtResizer.shared.resize(maxwidth, syspath(image_path))
+
+    with open(syspath(image_path), 'rb') as f:
+        data = f.read()
+    return mediafile.Image(data, type=mediafile.ImageType.front)
+
+
+# 'extractart' command.
+
 def extract(lib, outpath, query):
-    items = lib.items(query)
-    for i_item in items:
-        item = i_item
-        break
-    else:
-        log.error('No item matches query.')
+    item = lib.items(query).get()
+    if not item:
+        log.error(u'No item matches query.')
         return
 
     # Extract the art.
@@ -158,14 +159,14 @@ def extract(lib, outpath, query):
 
     art = mf.art
     if not art:
-        log.error('No album art present in %s - %s.' %
-                  (item.artist, item.title))
+        log.error(u'No album art present in {0} - {1}.'
+                  .format(item.artist, item.title))
         return
 
     # Add an extension to the filename.
     ext = imghdr.what(None, h=art)
     if not ext:
-        log.error('Unknown image type.')
+        log.error(u'Unknown image type.')
         return
     outpath += '.' + ext
 
@@ -174,13 +175,16 @@ def extract(lib, outpath, query):
     with open(syspath(outpath), 'wb') as f:
         f.write(art)
 
-# "clearart" command.
+
+# 'clearart' command.
+
 def clear(lib, query):
-    log.info('Clearing album art from items:')
+    log.info(u'Clearing album art from items:')
     for item in lib.items(query):
-        log.info(u'%s - %s' % (item.artist, item.title))
+        log.info(u'{0} - {1}'.format(item.artist, item.title))
         try:
-            mf = mediafile.MediaFile(syspath(item.path))
+            mf = mediafile.MediaFile(syspath(item.path),
+                                     config['id3v23'].get(bool))
         except mediafile.UnreadableFileError as exc:
             log.error(u'Could not clear art from {0}: {1}'.format(
                 displayable_path(item.path), exc
@@ -188,10 +192,3 @@ def clear(lib, query):
             continue
         mf.art = None
         mf.save()
-
-# Automatically embed art into imported albums.
-@EmbedCoverArtPlugin.listen('album_imported')
-def album_imported(lib, album):
-    if album.artpath and config['embedart']['auto']:
-        _embed(album.artpath, album.items(),
-               config['embedart']['maxwidth'].get(int))
