@@ -13,17 +13,21 @@
 # included in all copies or substantial portions of the Software.
 
 """Write paths of imported files in various formats to ease later import in a
-music player.
+music player. Also allow printing the new file locations to stdout in case
+one wants to manually add music to a player by its path.
 """
 import datetime
 import os
 import re
+import logging
 
 from beets.plugins import BeetsPlugin
 from beets.util import normpath, syspath, bytestring_path
 from beets import config
 
 M3U_DEFAULT_NAME = 'imported.m3u'
+log = logging.getLogger('beets')
+
 
 class ImportFeedsPlugin(BeetsPlugin):
     def __init__(self):
@@ -50,6 +54,7 @@ class ImportFeedsPlugin(BeetsPlugin):
         else:
             self.config['relative_to'] = feeds_dir
 
+
 def _get_feeds_dir(lib):
     """Given a Library object, return the path to the feeds directory to be
     used (either in the library directory or an explicitly configured
@@ -63,11 +68,12 @@ def _get_feeds_dir(lib):
         os.makedirs(syspath(dirpath))
     return dirpath
 
+
 def _build_m3u_filename(basename):
     """Builds unique m3u filename by appending given basename to current
     date."""
 
-    basename = re.sub(r"[\s,'\"]", '_', basename)
+    basename = re.sub(r"[\s,/\\'\"]", '_', basename)
     date = datetime.datetime.now().strftime("%Y%m%d_%Hh%M")
     path = normpath(os.path.join(
         config['importfeeds']['dir'].as_filename(),
@@ -75,6 +81,7 @@ def _build_m3u_filename(basename):
     ))
     return path
 
+
 def _write_m3u(m3u_path, items_paths):
     """Append relative paths to items into m3u file.
     """
@@ -82,13 +89,14 @@ def _write_m3u(m3u_path, items_paths):
         for path in items_paths:
             f.write(path + '\n')
 
+
 def _record_items(lib, basename, items):
     """Records relative paths to the given items for each feed format
     """
     feedsdir = bytestring_path(config['importfeeds']['dir'].as_filename())
     formats = config['importfeeds']['formats'].as_str_seq()
     relative_to = config['importfeeds']['relative_to'].get() \
-            or config['importfeeds']['dir'].as_filename()
+        or config['importfeeds']['dir'].as_filename()
     relative_to = bytestring_path(relative_to)
 
     paths = []
@@ -96,9 +104,13 @@ def _record_items(lib, basename, items):
         if config['importfeeds']['absolute_path']:
             paths.append(item.path)
         else:
-            paths.append(os.path.relpath(
-                item.path, relative_to
-            ))
+            try:
+                relpath = os.path.relpath(item.path, relative_to)
+            except ValueError:
+                # On Windows, it is sometimes not possible to construct a
+                # relative path (if the files are on different disks).
+                relpath = item.path
+            paths.append(relpath)
 
     if 'm3u' in formats:
         basename = bytestring_path(
@@ -117,15 +129,23 @@ def _record_items(lib, basename, items):
             if not os.path.exists(syspath(dest)):
                 os.symlink(syspath(path), syspath(dest))
 
+    if 'echo' in formats:
+        log.info("Location of imported music:")
+        for path in paths:
+            log.info("  " + path)
+
+
 @ImportFeedsPlugin.listen('library_opened')
 def library_opened(lib):
     if config['importfeeds']['dir'].get() is None:
         config['importfeeds']['dir'] = _get_feeds_dir(lib)
 
+
 @ImportFeedsPlugin.listen('album_imported')
 def album_imported(lib, album):
     _record_items(lib, album.album, album.items())
 
+
 @ImportFeedsPlugin.listen('item_imported')
 def item_imported(lib, item):
     _record_items(lib, item.title, [item])
