@@ -15,12 +15,8 @@
 """Gets genres for imported music based on Last.fm tags.
 
 Uses a provided whitelist file to determine which tags are valid genres.
-The genre whitelist can be specified like so in .beetsconfig:
-
-    [lastgenre]
-    whitelist=/path/to/genres.txt
-
-The included (default) genre list was produced by scraping Wikipedia.
+The included (default) genre list was originally produced by scraping Wikipedia
+and has been edited to remove some questionable entries.
 The scraper script used is available here:
 https://gist.github.com/1241307
 """
@@ -38,7 +34,6 @@ from beets import library
 log = logging.getLogger('beets')
 
 LASTFM = pylast.LastFMNetwork(api_key=plugins.LASTFM_KEY)
-C14N_TREE = os.path.join(os.path.dirname(__file__), 'genres-tree.yaml')
 
 PYLAST_EXCEPTIONS = (
     pylast.WSError,
@@ -47,78 +42,47 @@ PYLAST_EXCEPTIONS = (
 )
 
 
+def deduplicate(seq):
+    """Remove duplicates from sequence wile preserving order.
+    """
+    seen = set()
+    return [x for x in seq if x not in seen and not seen.add(x)]
+
+
 # Core genre identification routine.
 
-def _tags_for(obj):
-    """Given a pylast entity (album or track), returns a list of
-    tag names for that entity. Returns an empty list if the entity is
+def _tags_for(obj, min_weight=None):
+    """Given a pylast entity (album or track), return a list of
+    tag names for that entity. Return an empty list if the entity is
     not found or another error occurs.
+
+    If `min_weight` is specified, tags are filtered by weight.
     """
     try:
-        res = obj.get_top_tags()
+        # Work around an inconsistency in pylast where
+        # Album.get_top_tags() does not return TopItem instances.
+        # https://code.google.com/p/pylast/issues/detail?id=85
+        if isinstance(obj, pylast.Album):
+            res = super(pylast.Album, obj).get_top_tags()
+        else:
+            res = obj.get_top_tags()
     except PYLAST_EXCEPTIONS as exc:
-        log.debug(u'last.fm error: %s' % unicode(exc))
+        log.debug(u'last.fm error: {0}'.format(exc))
         return []
 
-    tags = []
-    for el in res:
-        if isinstance(el, pylast.TopItem):
-            el = el.item
-        tags.append(el.get_name())
-    log.debug(u'last.fm tags: %s' % unicode(tags))
-    return tags
+    # Filter by weight (optionally).
+    if min_weight:
+        res = [el for el in res if (el.weight or 0) >= min_weight]
 
-def _is_allowed(genre):
-    """Determine whether the genre is present in the whitelist,
-    returning a boolean.
-    """
-    if genre is None:
-        return False
-    if genre.lower() in options['whitelist']:
-        return True
-    return False
+    # Get strings from tags.
+    res = [el.item.get_name().lower() for el in res]
 
-def _find_allowed(genres):
-    """Given a list of candidate genres (strings), return an allowed
-    genre string. If `multiple` is on, then this may be a
-    comma-separated list; otherwise, it is one of the elements of
-    `genres`.
-    """
-    allowed_genres = [g.title() for g in genres if _is_allowed(g)]
-    if not allowed_genres:
-        return
-    if config['lastgenre']['multiple']:
-        return u', '.join(allowed_genres)
-    else:
-        return allowed_genres[0]
-
-def _strings_to_genre(tags):
-    """Given a list of strings, return a genre. Returns the first string
-    that is present in the genre whitelist (or the canonicalization
-    tree) or None if no tag is suitable.
-    """
-    if not tags:
-        return None
-    elif not options['whitelist']:
-        return tags[0].title()
-
-    if options.get('c14n'):
-        # Use the canonicalization tree.
-        for tag in tags:
-            return _find_allowed(find_parents(tag, options['branches']))
-    else:
-        # Just use the flat whitelist.
-        return _find_allowed(tags)
-
-def fetch_genre(lastfm_obj):
-    """Return the genre for a pylast entity or None if no suitable genre
-    can be found. Ex. 'Electronic, House, Dance'
-    """
-    return _strings_to_genre(_tags_for(lastfm_obj))
+    return res
 
 
 # Canonicalization tree processing.
 
+
 def flatten_tree(elem, path, branches):
     """Flatten nested lists/dictionaries into lists of strings
     (branches).
@@ -135,6 +99,7 @@ def flatten_tree(elem, path, branches):
     else:
         branches.append(path + [unicode(elem)])
 
+
 def find_parents(candidate, branches):
     """Find parents genre of a given genre, ordered from the closest to
     the further parent.
@@ -148,96 +113,60 @@ def find_parents(candidate, branches):
     return [candidate]
 
 
-# Cached entity lookups.
-
-_genre_cache = {}
-
-def _cached_lookup(entity, method, *args):
-    """Get a genre based on the named entity using the callable `method`
-    whose arguments are given in the sequence `args`. The genre lookup
-    is cached based on the entity name and the arguments.
-    """
-    # Shortcut if we're missing metadata.
-    if any(not s for s in args):
-        return None
-
-    key = u'{0}.{1}'.format(entity, u'-'.join(unicode(a) for a in args))
-    if key in _genre_cache:
-        return _genre_cache[key]
-    else:
-        genre = fetch_genre(method(*args))
-        _genre_cache[key] = genre
-        return genre
-
-def fetch_album_genre(obj):
-    """Return the album genre for this Item or Album.
-    """
-    return _cached_lookup(u'album', LASTFM.get_album, obj.albumartist,
-                          obj.album)
-
-def fetch_album_artist_genre(obj):
-    """Return the album artist genre for this Item or Album.
-    """
-    return _cached_lookup(u'artist', LASTFM.get_artist, obj.albumartist)
-
-def fetch_artist_genre(item):
-    """Returns the track artist genre for this Item.
-    """
-    return _cached_lookup(u'artist', LASTFM.get_artist, item.artist)
-
-def fetch_track_genre(obj):
-    """Returns the track genre for this Item.
-    """
-    return _cached_lookup(u'track', LASTFM.get_track, obj.artist, obj.title)
-
-
 # Main plugin logic.
 
-options = {
-    'whitelist': None,
-    'branches': None,
-    'c14n': False,
-}
+WHITELIST = os.path.join(os.path.dirname(__file__), 'genres.txt')
+C14N_TREE = os.path.join(os.path.dirname(__file__), 'genres-tree.yaml')
+
+
 class LastGenrePlugin(plugins.BeetsPlugin):
     def __init__(self):
         super(LastGenrePlugin, self).__init__()
 
         self.config.add({
-            'whitelist': os.path.join(os.path.dirname(__file__), 'genres.txt'),
-            'multiple': False,
+            'whitelist': True,
+            'min_weight': 10,
+            'count': 1,
             'fallback': None,
-            'canonical': None,
+            'canonical': False,
             'source': 'album',
             'force': True,
             'auto': True,
+            'separator': u', ',
         })
 
+        self.setup()
+
+    def setup(self):
+        """Setup plugin from config options
+        """
         if self.config['auto']:
             self.import_stages = [self.imported]
 
-        # Read the whitelist file.
-        wl_filename = self.config['whitelist'].as_filename()
-        whitelist = set()
-        with open(wl_filename) as f:
-            for line in f:
-                line = line.decode('utf8').strip().lower()
-                if line:
-                    whitelist.add(line)
-        options['whitelist'] = whitelist
+        self._genre_cache = {}
+
+        # Read the whitelist file if enabled.
+        self.whitelist = set()
+        wl_filename = self.config['whitelist'].get()
+        if wl_filename in (True, ''):  # Indicates the default whitelist.
+            wl_filename = WHITELIST
+        if wl_filename:
+            wl_filename = normpath(wl_filename)
+            with open(wl_filename, 'r') as f:
+                for line in f:
+                    line = line.decode('utf8').strip().lower()
+                    if line and not line.startswith(u'#'):
+                        self.whitelist.add(line)
 
         # Read the genres tree for canonicalization if enabled.
+        self.c14n_branches = []
         c14n_filename = self.config['canonical'].get()
-        if c14n_filename is not None:
-            c14n_filename = c14n_filename.strip()
-            if not c14n_filename:
-                c14n_filename = C14N_TREE
+        if c14n_filename in (True, ''):  # Default tree.
+            c14n_filename = C14N_TREE
+        if c14n_filename:
             c14n_filename = normpath(c14n_filename)
-
             genres_tree = yaml.load(open(c14n_filename, 'r'))
-            branches = []
-            flatten_tree(genres_tree, [], branches)
-            options['branches'] = branches
-            options['c14n'] = True
+            flatten_tree(genres_tree, [], self.c14n_branches)
 
     @property
     def sources(self):
@@ -252,6 +181,100 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         elif source == 'artist':
             return 'artist',
 
+    def _resolve_genres(self, tags):
+        """Given a list of strings, return a genre by joining them into a
+        single string and (optionally) canonicalizing each.
+        """
+        if not tags:
+            return None
+
+        count = self.config['count'].get(int)
+        if self.c14n_branches:
+            # Extend the list to consider tags parents in the c14n tree
+            tags_all = []
+            for tag in tags:
+                # Add parents that are in the whitelist, or add the oldest
+                # ancestor if no whitelist
+                if self.whitelist:
+                    parents = [x for x in find_parents(tag, self.c14n_branches)
+                               if self._is_allowed(x)]
+                else:
+                    parents = [find_parents(tag, self.c14n_branches)[-1]]
+
+                tags_all += parents
+                if len(tags_all) >= count:
+                    break
+            tags = tags_all
+
+        tags = deduplicate(tags)
+
+        # c14n only adds allowed genres but we may have had forbidden genres in
+        # the original tags list
+        tags = [x.title() for x in tags if self._is_allowed(x)]
+
+        return self.config['separator'].get(unicode).join(
+            tags[:self.config['count'].get(int)]
+        )
+
+    def fetch_genre(self, lastfm_obj):
+        """Return the genre for a pylast entity or None if no suitable genre
+        can be found. Ex. 'Electronic, House, Dance'
+        """
+        min_weight = self.config['min_weight'].get(int)
+        return self._resolve_genres(_tags_for(lastfm_obj, min_weight))
+
+    def _is_allowed(self, genre):
+        """Determine whether the genre is present in the whitelist,
+        returning a boolean.
+        """
+        if genre is None:
+            return False
+        if not self.whitelist or genre in self.whitelist:
+            return True
+        return False
+
+    # Cached entity lookups.
+
+    def _cached_lookup(self, entity, method, *args):
+        """Get a genre based on the named entity using the callable `method`
+        whose arguments are given in the sequence `args`. The genre lookup
+        is cached based on the entity name and the arguments.
+        """
+        # Shortcut if we're missing metadata.
+        if any(not s for s in args):
+            return None
+
+        key = u'{0}.{1}'.format(entity, u'-'.join(unicode(a) for a in args))
+        if key in self._genre_cache:
+            return self._genre_cache[key]
+        else:
+            genre = self.fetch_genre(method(*args))
+            self._genre_cache[key] = genre
+            return genre
+
+    def fetch_album_genre(self, obj):
+        """Return the album genre for this Item or Album.
+        """
+        return self._cached_lookup(u'album', LASTFM.get_album, obj.albumartist,
+                                   obj.album)
+
+    def fetch_album_artist_genre(self, obj):
+        """Return the album artist genre for this Item or Album.
+        """
+        return self._cached_lookup(u'artist', LASTFM.get_artist,
+                                   obj.albumartist)
+
+    def fetch_artist_genre(self, item):
+        """Returns the track artist genre for this Item.
+        """
+        return self._cached_lookup(u'artist', LASTFM.get_artist, item.artist)
+
+    def fetch_track_genre(self, obj):
+        """Returns the track genre for this Item.
+        """
+        return self._cached_lookup(u'track', LASTFM.get_track, obj.artist,
+                                   obj.title)
+
     def _get_genre(self, obj):
         """Get the genre string for an Album or Item object based on
         self.sources. Return a `(genre, source)` pair. The
@@ -263,20 +286,21 @@ class LastGenrePlugin(plugins.BeetsPlugin):
             - fallback
             - None
         """
+
         # Shortcut to existing genre if not forcing.
-        if not self.config['force'] and _is_allowed(obj.genre):
+        if not self.config['force'] and self._is_allowed(obj.genre):
             return obj.genre, 'keep'
 
         # Track genre (for Items only).
         if isinstance(obj, library.Item):
             if 'track' in self.sources:
-                result = fetch_track_genre(obj)
+                result = self.fetch_track_genre(obj)
                 if result:
                     return result, 'track'
 
         # Album genre.
         if 'album' in self.sources:
-            result = fetch_album_genre(obj)
+            result = self.fetch_album_genre(obj)
             if result:
                 return result, 'album'
 
@@ -284,18 +308,18 @@ class LastGenrePlugin(plugins.BeetsPlugin):
         if 'artist' in self.sources:
             result = None
             if isinstance(obj, library.Item):
-                result = fetch_artist_genre(obj)
+                result = self.fetch_artist_genre(obj)
             elif obj.albumartist != 'Various Artists':
-                result = fetch_album_artist_genre(obj)
+                result = self.fetch_album_artist_genre(obj)
             else:
                 # For "Various Artists", pick the most popular track genre.
                 item_genres = []
                 for item in obj.items():
                     item_genre = None
                     if 'track' in self.sources:
-                        item_genre = fetch_track_genre(item)
+                        item_genre = self.fetch_track_genre(item)
                     if not item_genre:
-                        item_genre = fetch_artist_genre(item)
+                        item_genre = self.fetch_artist_genre(item)
                     if item_genre:
                         item_genres.append(item_genre)
                 if item_genres:
@@ -306,7 +330,7 @@ class LastGenrePlugin(plugins.BeetsPlugin):
 
         # Filter the existing genre.
         if obj.genre:
-            result = _strings_to_genre([obj.genre])
+            result = self._resolve_genres([obj.genre])
             if result:
                 return result, 'original'
 
@@ -319,13 +343,15 @@ class LastGenrePlugin(plugins.BeetsPlugin):
 
     def commands(self):
         lastgenre_cmd = ui.Subcommand('lastgenre', help='fetch genres')
-        lastgenre_cmd.parser.add_option('-f', '--force', dest='force',
-                              action='store_true',
-                              default=False,
-                              help='re-download genre when already present')
-        lastgenre_cmd.parser.add_option('-s', '--source', dest='source',
-                              type='string',
-                              help='genre source: artist, album, or track')
+        lastgenre_cmd.parser.add_option(
+            '-f', '--force', dest='force', action='store_true', default=False,
+            help='re-download genre when already present'
+        )
+        lastgenre_cmd.parser.add_option(
+            '-s', '--source', dest='source', type='string',
+            help='genre source: artist, album, or track'
+        )
+
         def lastgenre_func(lib, opts, args):
             write = config['import']['write'].get(bool)
             self.config.set_args(opts)
@@ -343,12 +369,12 @@ class LastGenrePlugin(plugins.BeetsPlugin):
                     if 'track' in self.sources:
                         item.genre, src = self._get_genre(item)
                         item.store()
-                        log.info(u'genre for track {0} - {1} ({2}): {3}'.format(
-                            item.artist, item.title, src, item.genre
-                        ))
+                        log.info(u'genre for track {0} - {1} ({2}): {3}'
+                                 .format(item.artist, item.title, src,
+                                         item.genre))
 
                     if write:
-                        item.write()
+                        item.try_write()
 
         lastgenre_cmd.func = lastgenre_func
         return [lastgenre_cmd]
@@ -356,22 +382,22 @@ class LastGenrePlugin(plugins.BeetsPlugin):
     def imported(self, session, task):
         """Event hook called when an import task finishes."""
         if task.is_album:
-            album = session.lib.get_album(task.album_id)
+            album = task.album
             album.genre, src = self._get_genre(album)
             log.debug(u'added last.fm album genre ({0}): {1}'.format(
-                  src, album.genre))
+                src, album.genre))
             album.store()
 
             if 'track' in self.sources:
                 for item in album.items():
                     item.genre, src = self._get_genre(item)
                     log.debug(u'added last.fm item genre ({0}): {1}'.format(
-                          src, item.genre))
+                        src, item.genre))
                     item.store()
 
         else:
             item = task.item
             item.genre, src = self._get_genre(item)
             log.debug(u'added last.fm item genre ({0}): {1}'.format(
-                  src, item.genre))
+                src, item.genre))
             item.store()
