@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2014, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,6 +22,7 @@ import urllib
 import json
 import unicodedata
 import difflib
+import itertools
 
 from beets.plugins import BeetsPlugin
 from beets import ui
@@ -64,17 +65,20 @@ def fetch_url(url):
         log.debug(u'failed to fetch: {0} ({1})'.format(url, unicode(exc)))
         return None
 
+
 def unescape(text):
     """Resolves &#xxx; HTML entities (and some others)."""
     if isinstance(text, str):
         text = text.decode('utf8', 'ignore')
     out = text.replace(u'&nbsp;', u' ')
+
     def replchar(m):
         num = m.group(1)
         return unichr(int(num))
     out = re.sub(u"&#(\d+);", replchar, out)
     return out
 
+
 def extract_text(html, starttag):
     """Extract the text from a <DIV> tag in the HTML starting with
     ``starttag``. Returns None if parsing fails.
@@ -90,11 +94,11 @@ def extract_text(html, starttag):
     parts = []
     pos = 0
     for match in DIV_RE.finditer(html):
-        if match.group(1): # Closing tag.
+        if match.group(1):  # Closing tag.
             level -= 1
             if level == 0:
                 pos = match.end()
-        else: # Opening tag.
+        else:  # Opening tag.
             if level == 0:
                 parts.append(html[pos:match.start()])
 
@@ -109,6 +113,7 @@ def extract_text(html, starttag):
     lyrics = ''.join(parts)
     return strip_cruft(lyrics)
 
+
 def strip_cruft(lyrics, wscollapse=True):
     """Clean up HTML from an extracted lyrics string. For example, <BR>
     tags are replaced with newlines.
@@ -116,15 +121,64 @@ def strip_cruft(lyrics, wscollapse=True):
     lyrics = COMMENT_RE.sub('', lyrics)
     lyrics = unescape(lyrics)
     if wscollapse:
-        lyrics = re.sub(r'\s+', ' ', lyrics) # Whitespace collapse.
-    lyrics = BREAK_RE.sub('\n', lyrics) # <BR> newlines.
+        lyrics = re.sub(r'\s+', ' ', lyrics)  # Whitespace collapse.
+    lyrics = re.sub(r'<(script).*?</\1>(?s)', '', lyrics)  # Strip script tags.
+    lyrics = BREAK_RE.sub('\n', lyrics)  # <BR> newlines.
     lyrics = re.sub(r'\n +', '\n', lyrics)
     lyrics = re.sub(r' +\n', '\n', lyrics)
-    lyrics = TAG_RE.sub('', lyrics) # Strip remaining HTML tags.
-    lyrics = lyrics.replace('\r','\n')
+    lyrics = TAG_RE.sub('', lyrics)  # Strip remaining HTML tags.
+    lyrics = lyrics.replace('\r', '\n')
     lyrics = lyrics.strip()
     return lyrics
 
+
+def search_pairs(item):
+    """Yield a pairs of artists and titles to search for.
+
+    The first item in the pair is the name of the artist, the second
+    item is a list of song names.
+
+    In addition to the artist and title obtained from the `item` the
+    method tries to strip extra information like paranthesized suffixes
+    and featured artists from the strings and add them as caniddates.
+    The method also tries to split multiple titles separated with `/`.
+    """
+
+    title, artist = item.title, item.artist
+    titles = [title]
+    artists = [artist]
+
+    # Remove any featuring artists from the artists name
+    pattern = r"(.*?) (&|\b(and|ft|feat(uring)?\b))"
+    match = re.search(pattern, artist, re.IGNORECASE)
+    if match:
+        artists.append(match.group(1))
+
+    # Remove a parenthesized suffix from a title string. Common
+    # examples include (live), (remix), and (acoustic).
+    pattern = r"(.+?)\s+[(].*[)]$"
+    match = re.search(pattern, title, re.IGNORECASE)
+    if match:
+        titles.append(match.group(1))
+
+    # Remove any featuring artists from the title
+    pattern = r"(.*?) \b(ft|feat(uring)?)\b"
+    for title in titles:
+        match = re.search(pattern, title, re.IGNORECASE)
+        if match:
+            titles.append(match.group(1))
+
+    # Check for a dual song (e.g. Pink Floyd - Speak to Me / Breathe)
+    # and each of them.
+    multi_titles = []
+    for title in titles:
+        multi_titles.append([title])
+        if '/' in title:
+            multi_titles.append([x.strip() for x in title.split('/')])
+
+    return itertools.product(artists, multi_titles)
+
+
 def _encode(s):
     """Encode the string for inclusion in a URL (common to both
     LyricsWiki and Lyrics.com).
@@ -139,6 +193,8 @@ def _encode(s):
 # LyricsWiki.
 
 LYRICSWIKI_URL_PATTERN = 'http://lyrics.wikia.com/%s:%s'
+
+
 def _lw_encode(s):
     s = re.sub(r'\s+', '_', s)
     s = s.replace("<", "Less_Than")
@@ -148,6 +204,7 @@ def _lw_encode(s):
     s = re.sub(r'[\]\}]', ')', s)
     return _encode(s)
 
+
 def fetch_lyricswiki(artist, title):
     """Fetch lyrics from LyricsWiki."""
     url = LYRICSWIKI_URL_PATTERN % (_lw_encode(artist), _lw_encode(title))
@@ -167,11 +224,14 @@ LYRICSCOM_NOT_FOUND = (
     'Sorry, we do not have the lyric',
     'Submit Lyrics',
 )
+
+
 def _lc_encode(s):
     s = re.sub(r'[^\w\s-]', '', s)
     s = re.sub(r'\s+', '-', s)
     return _encode(s).lower()
 
+
 def fetch_lyricscom(artist, title):
     """Fetch lyrics from Lyrics.com."""
     url = LYRICSCOM_URL_PATTERN % (_lc_encode(title), _lc_encode(artist))
@@ -196,22 +256,21 @@ def fetch_lyricscom(artist, title):
 def slugify(text):
     """Normalize a string and remove non-alphanumeric characters.
     """
-    # http://stackoverflow.com/questions/295135/turn-a-string-into-a-valid-
-    # filename-in-python
-
-    # Remove content within parentheses
-    pat = "([^,\(]*)\((.*?)\)"
-    text = re.sub(pat,'\g<1>', text).strip()
+    text = re.sub(r"[-'_\s]", '_', text)
+    text = re.sub(r"_+", '_', text).strip('_')
+    pat = "([^,\(]*)\((.*?)\)"  # Remove content within parentheses
+    text = re.sub(pat, '\g<1>', text).strip()
     try:
         text = unicodedata.normalize('NFKD', text).encode('ascii', 'ignore')
         text = unicode(re.sub('[-\s]+', ' ', text))
     except UnicodeDecodeError:
-        log.exception("Failing to normalize '%s'" % (text))
-    return urllib.quote(text)
+        log.exception(u"Failing to normalize '{0}'".format(text))
+    return text
 
 
-BY_TRANS     = ['by', 'par']
-LYRICS_TRANS = ['lyrics', 'paroles']
+BY_TRANS = ['by', 'par', 'de', 'von']
+LYRICS_TRANS = ['lyrics', 'paroles', 'letras', 'liedtexte']
+
 
 def is_page_candidate(urlLink, urlTitle, title, artist):
     """Return True if the URL title makes it a good candidate to be a
@@ -227,13 +286,14 @@ def is_page_candidate(urlLink, urlTitle, title, artist):
         return True
     # or try extracting song title from URL title and check if
     # they are close enough
-    tokens = [by+'%20'+artist for by in BY_TRANS] + \
-             [artist, sitename, sitename.replace('www.','')] + LYRICS_TRANS
-    songTitle = re.sub(u'(%s)' % u'|'.join(tokens) ,u'', urlTitle).strip('%20')
+    tokens = [by + '_' + artist for by in BY_TRANS] + \
+             [artist, sitename, sitename.replace('www.', '')] + LYRICS_TRANS
+    songTitle = re.sub(u'(%s)' % u'|'.join(tokens), u'', urlTitle)
 
     typoRatio = .8
     return difflib.SequenceMatcher(None, songTitle, title).ratio() >= typoRatio
 
+
 def insert_line_feeds(text):
     """Insert newlines before upper-case characters.
     """
@@ -243,6 +303,7 @@ def insert_line_feeds(text):
         tokensStr[idx] = ltoken[0] + '\n' + ltoken[1]
     return ''.join(tokensStr)
 
+
 def sanitize_lyrics(text):
     """Clean text, returning raw lyrics as output or None if it happens
     that input text is actually not lyrics content.  Clean (x)html tags
@@ -254,7 +315,7 @@ def sanitize_lyrics(text):
     if '\n' not in text:
         text = insert_line_feeds(text)
 
-    while text.count('\n\n') > text.count('\n')/4:
+    while text.count('\n\n') > text.count('\n') // 4:
         # Remove first occurrence of \n for each sequence of \n
         text = re.sub(r'\n(\n+)', '\g<1>', text)
 
@@ -262,40 +323,62 @@ def sanitize_lyrics(text):
 
     return text
 
-def is_lyrics(text, artist):
+
+def remove_credits(text):
+    """Remove first/last line of text if it contains the word 'lyrics'
+    eg 'Lyrics by songsdatabase.com'
+    """
+    textlines = text.split('\n')
+    credits = None
+    for i in (0, -1):
+        if textlines and 'lyrics' in textlines[i].lower():
+            credits = textlines.pop(i)
+    if credits:
+        text = '\n'.join(textlines)
+    return text
+
+
+def is_lyrics(text, artist=None):
     """Determine whether the text seems to be valid lyrics.
     """
-    badTriggers = []
+    if not text:
+        return
+
+    badTriggersOcc = []
     nbLines = text.count('\n')
     if nbLines <= 1:
-        log.debug("Ignoring too short lyrics '%s'" % text)
+        log.debug(u"Ignoring too short lyrics '{0}'".format(text))
         return 0
     elif nbLines < 5:
-        badTriggers.append('too_short')
+        badTriggersOcc.append('too_short')
     else:
-        # Don't penalize long text because of lyrics keyword in credits
-        textlines = text.split('\n')
-        popped = False
-        for i in [len(textlines)-1, 0]:
-            if 'lyrics' in textlines[i].lower():
-                popped = textlines.pop(i)
-        if popped:
-            text = '\n'.join(textlines)
+        # Lyrics look legit, remove credits to avoid being penalized further
+        # down
+        text = remove_credits(text)
 
-    for item in artist, 'lyrics', 'copyright', 'property':
-        badTriggers += [item] * len(re.findall(r'\W%s\W' % item, text, re.I))
+    badTriggers = ['lyrics', 'copyright', 'property']
+    if artist:
+        badTriggersOcc += [artist]
 
-    if badTriggers:
-        log.debug('Bad triggers detected: %s' % badTriggers)
+    for item in badTriggers:
+        badTriggersOcc += [item] * len(re.findall(r'\W%s\W' % item,
+                                                  text, re.I))
+
+    if badTriggersOcc:
+        log.debug(u'Bad triggers detected: {0}'.format(badTriggersOcc))
+
+    return len(badTriggersOcc) < 2
 
-    return len(badTriggers) < 2
 
 def scrape_lyrics_from_url(url):
     """Scrape lyrics from a URL. If no lyrics can be found, return None
     instead.
     """
-    from bs4 import BeautifulSoup, Tag, Comment
+    from bs4 import BeautifulSoup, Comment
     html = fetch_url(url)
+    if not html:
+        return None
+
     soup = BeautifulSoup(html)
 
     for tag in soup.findAll('br'):
@@ -303,7 +386,7 @@ def scrape_lyrics_from_url(url):
 
     # Remove non relevant html parts
     [s.extract() for s in soup(['head', 'script'])]
-    comments = soup.findAll(text=lambda text:isinstance(text, Comment))
+    comments = soup.findAll(text=lambda text: isinstance(text, Comment))
     [s.extract() for s in comments]
 
     try:
@@ -311,8 +394,8 @@ def scrape_lyrics_from_url(url):
             tag.name = 'p'          # keep tag contents
 
     except Exception, e:
-        log.debug('Error %s when replacing containing marker by p marker' % e,
-            exc_info=True)
+        log.debug(u'Error {0} when replacing containing marker by p marker'
+                  .format(e, exc_info=True))
 
     # Make better soup from current soup! The previous unclosed <p> sections
     # are now closed.  Use str() rather than prettify() as it's more
@@ -340,6 +423,7 @@ def scrape_lyrics_from_url(url):
         soup = BeautifulSoup(tagTokens[0])
         return unescape(tagTokens[0].strip("\n\r: "))
 
+
 def fetch_google(artist, title):
     """Fetch lyrics from Google search results.
     """
@@ -353,8 +437,8 @@ def fetch_google(artist, title):
     data = json.load(data)
     if 'error' in data:
         reason = data['error']['errors'][0]['reason']
-        log.debug(u'google lyrics backend error: %s' % reason)
-        return None
+        log.debug(u'google lyrics backend error: {0}'.format(reason))
+        return
 
     if 'items' in data.keys():
         for item in data['items']:
@@ -369,12 +453,13 @@ def fetch_google(artist, title):
             lyrics = sanitize_lyrics(lyrics)
 
             if is_lyrics(lyrics, artist):
-                log.debug(u'got lyrics from %s' % item['displayLink'])
+                log.debug(u'got lyrics from {0}'.format(item['displayLink']))
                 return lyrics
 
 
 # Plugin logic.
 
+
 class LyricsPlugin(BeetsPlugin):
     def __init__(self):
         super(LyricsPlugin, self).__init__()
@@ -396,72 +481,78 @@ class LyricsPlugin(BeetsPlugin):
         cmd.parser.add_option('-p', '--print', dest='printlyr',
                               action='store_true', default=False,
                               help='print lyrics to console')
+        cmd.parser.add_option('-f', '--force', dest='force_refetch',
+                              action='store_true', default=False,
+                              help='always re-download lyrics')
+
         def func(lib, opts, args):
             # The "write to files" option corresponds to the
             # import_write config value.
             write = config['import']['write'].get(bool)
             for item in lib.items(ui.decargs(args)):
-                self.fetch_item_lyrics(lib, logging.INFO, item, write)
+                self.fetch_item_lyrics(lib, logging.INFO, item, write,
+                                       opts.force_refetch)
                 if opts.printlyr and item.lyrics:
                     ui.print_(item.lyrics)
+
         cmd.func = func
         return [cmd]
 
-    # Auto-fetch lyrics on import.
     def imported(self, session, task):
+        """Import hook for fetching lyrics automatically.
+        """
         if self.config['auto']:
             for item in task.imported_items():
-                self.fetch_item_lyrics(session.lib, logging.DEBUG, item, False)
+                self.fetch_item_lyrics(session.lib, logging.DEBUG, item,
+                                       False, False)
 
-    def fetch_item_lyrics(self, lib, loglevel, item, write):
+    def fetch_item_lyrics(self, lib, loglevel, item, write, force):
         """Fetch and store lyrics for a single item. If ``write``, then the
         lyrics will also be written to the file itself. The ``loglevel``
         parameter controls the visibility of the function's status log
         messages.
         """
-        fallback = self.config['fallback'].get()
-
         # Skip if the item already has lyrics.
-        if item.lyrics:
-            log.log(loglevel, u'lyrics already present: %s - %s' %
-                              (item.artist, item.title))
+        if not force and item.lyrics:
+            log.log(loglevel, u'lyrics already present: {0} - {1}'
+                    .format(item.artist, item.title))
             return
 
-        # Fetch lyrics.
-        lyrics = self.get_lyrics(item.artist, item.title)
-        if not lyrics:
-            log.log(loglevel, u'lyrics not found: %s - %s' %
-                              (item.artist, item.title))
+        lyrics = None
+        for artist, titles in search_pairs(item):
+            lyrics = [self.get_lyrics(artist, title) for title in titles]
+            if any(lyrics):
+                break
+
+        lyrics = u"\n\n---\n\n".join([l for l in lyrics if l])
+
+        if lyrics:
+            log.log(loglevel, u'fetched lyrics: {0} - {1}'
+                              .format(item.artist, item.title))
+        else:
+            log.log(loglevel, u'lyrics not found: {0} - {1}'
+                              .format(item.artist, item.title))
+            fallback = self.config['fallback'].get()
             if fallback:
                 lyrics = fallback
             else:
                 return
-        else:
-            log.log(loglevel, u'fetched lyrics: %s - %s' %
-                              (item.artist, item.title))
 
         item.lyrics = lyrics
 
         if write:
-            item.write()
+            item.try_write()
         item.store()
 
     def get_lyrics(self, artist, title):
         """Fetch lyrics, trying each source in turn. Return a string or
         None if no lyrics were found.
         """
-        # Remove featuring artists from search.
-        pattern = u"(.*) feat(uring|\.)?\s\S+"
-        match = re.search(pattern, artist, re.IGNORECASE)
-        if match:
-            artist = match.group(0)
-
         for backend in self.backends:
             lyrics = backend(artist, title)
             if lyrics:
                 if isinstance(lyrics, str):
                     lyrics = lyrics.decode('utf8', 'ignore')
-                log.debug(u'got lyrics from backend: {0}'.format(
-                    backend.__name__
-                ))
-                return lyrics
+                log.debug(u'got lyrics from backend: {0}'
+                          .format(backend.__name__))
+                return lyrics.strip()
