@@ -17,7 +17,7 @@
 import logging
 
 from beets.autotag import hooks
-from beets.library import Item, Album
+from beets.library import Item
 from beets.plugins import BeetsPlugin
 from beets.ui import decargs, print_obj, Subcommand
 
@@ -28,7 +28,7 @@ log = logging.getLogger('beets')
 def _missing_count(album):
     """Return number of missing items in `album`.
     """
-    return album.tracktotal - len([i for i in album.items()])
+    return (album.tracktotal or 0) - len(album.items())
 
 
 def _missing(album):
@@ -43,7 +43,7 @@ def _missing(album):
         for track_info in getattr(album_info, 'tracks', []):
             if track_info.track_id not in item_mbids:
                 item = _item(track_info, album_info, album.id)
-                log.debug('{0}: track {1} in album {2}'
+                log.debug(u'{0}: track {1} in album {2}'
                           .format(PLUGIN,
                                   track_info.track_id,
                                   album_info.album_id))
@@ -61,41 +61,41 @@ def _item(track_info, album_info, album_id):
     t = track_info
     a = album_info
 
-    return Item(
-        album_id =           album_id,
-        album =              a.album,
-        albumartist =        a.artist,
-        albumartist_credit = a.artist_credit,
-        albumartist_sort =   a.artist_sort,
-        albumdisambig =      a.albumdisambig,
-        albumstatus =        a.albumstatus,
-        albumtype =          a.albumtype,
-        artist =             t.artist,
-        artist_credit =      t.artist_credit,
-        artist_sort =        t.artist_sort,
-        asin =               a.asin,
-        catalognum =         a.catalognum,
-        comp =               a.va,
-        country =            a.country,
-        day =                a.day,
-        disc =               t.medium,
-        disctitle =          t.disctitle,
-        disctotal =          a.mediums,
-        label =              a.label,
-        language =           a.language,
-        length =             t.length,
-        mb_albumid =         a.album_id,
-        mb_artistid =        t.artist_id,
-        mb_releasegroupid =  a.releasegroup_id,
-        mb_trackid =         t.track_id,
-        media =              a.media,
-        month =              a.month,
-        script =             a.script,
-        title =              t.title,
-        track =              t.index,
-        tracktotal =         len(a.tracks),
-        year =               a.year,
-    )
+    return Item(**{
+        'album_id':           album_id,
+        'album':              a.album,
+        'albumartist':        a.artist,
+        'albumartist_credit': a.artist_credit,
+        'albumartist_sort':   a.artist_sort,
+        'albumdisambig':      a.albumdisambig,
+        'albumstatus':        a.albumstatus,
+        'albumtype':          a.albumtype,
+        'artist':             t.artist,
+        'artist_credit':      t.artist_credit,
+        'artist_sort':        t.artist_sort,
+        'asin':               a.asin,
+        'catalognum':         a.catalognum,
+        'comp':               a.va,
+        'country':            a.country,
+        'day':                a.day,
+        'disc':               t.medium,
+        'disctitle':          t.disctitle,
+        'disctotal':          a.mediums,
+        'label':              a.label,
+        'language':           a.language,
+        'length':             t.length,
+        'mb_albumid':         a.album_id,
+        'mb_artistid':        t.artist_id,
+        'mb_releasegroupid':  a.releasegroup_id,
+        'mb_trackid':         t.track_id,
+        'media':              a.media,
+        'month':              a.month,
+        'script':             a.script,
+        'title':              t.title,
+        'track':              t.index,
+        'tracktotal':         len(a.tracks),
+        'year':               a.year,
+    })
 
 
 class MissingPlugin(BeetsPlugin):
