@@ -15,7 +15,7 @@
 """Updates an MPD index whenever the library is changed.
 
 Put something like the following in your config.yaml to configure:
-    mpdupdate:
+    mpd:
         host: localhost
         port: 6600
         password: seekrit
@@ -31,6 +31,7 @@ from beets import config
 # once before beets exits.
 database_changed = False
 
+
 # No need to introduce a dependency on an MPD library for such a
 # simple use case. Here's a simple socket abstraction to make things
 # easier.
@@ -64,6 +65,7 @@ class BufferedSocket(object):
     def close(self):
         self.sock.close()
 
+
 def update_mpd(host='localhost', port=6600, password=None):
     """Sends the "update" command to the MPD server indicated,
     possibly authenticating with a password first.
@@ -94,15 +96,22 @@ def update_mpd(host='localhost', port=6600, password=None):
     s.close()
     print('... updated.')
 
+
 class MPDUpdatePlugin(BeetsPlugin):
     def __init__(self):
         super(MPDUpdatePlugin, self).__init__()
-        self.config.add({
-            'host': u'localhost',
-            'port': 6600,
+        config['mpd'].add({
+            'host':     u'localhost',
+            'port':     6600,
             'password': u'',
         })
 
+        # For backwards compatibility, use any values from the
+        # plugin-specific "mpdupdate" section.
+        for key in config['mpd'].keys():
+            if self.config[key].exists():
+                config['mpd'][key] = self.config[key].get()
+
 
 @MPDUpdatePlugin.listen('database_change')
 def handle_change(lib=None):
@@ -114,7 +123,7 @@ def handle_change(lib=None):
 def update(lib=None):
     if database_changed:
         update_mpd(
-            config['mpdupdate']['host'].get(unicode),
-            config['mpdupdate']['port'].get(int),
-            config['mpdupdate']['password'].get(unicode),
+            config['mpd']['host'].get(unicode),
+            config['mpd']['port'].get(int),
+            config['mpd']['password'].get(unicode),
         )
