@@ -19,7 +19,9 @@ from beets import util
 import beets.library
 import flask
 from flask import g
+from werkzeug.routing import BaseConverter, PathConverter
 import os
+import json
 
 
 # Utilities.
@@ -50,9 +52,109 @@ def _rep(obj, expand=False):
         return out
 
 
+def json_generator(items, root):
+    """Generator that dumps list of beets Items or Albums as JSON
+
+    :param root:  root key for JSON
+    :param items: list of :class:`Item` or :class:`Album` to dump
+    :returns:     generator that yields strings
+    """
+    yield '{"%s":[' % root
+    first = True
+    for item in items:
+        if first:
+            first = False
+        else:
+            yield ','
+        yield json.dumps(_rep(item))
+    yield ']}'
+
+
+def resource(name):
+    """Decorates a function to handle RESTful HTTP requests for a resource.
+    """
+    def make_responder(retriever):
+        def responder(ids):
+            entities = [retriever(id) for id in ids]
+            entities = [entity for entity in entities if entity]
+
+            if len(entities) == 1:
+                return flask.jsonify(_rep(entities[0]))
+            elif entities:
+                return app.response_class(
+                    json_generator(entities, root=name),
+                    mimetype='application/json'
+                )
+            else:
+                return flask.abort(404)
+        responder.__name__ = 'get_%s' % name
+        return responder
+    return make_responder
+
+
+def resource_query(name):
+    """Decorates a function to handle RESTful HTTP queries for resources.
+    """
+    def make_responder(query_func):
+        def responder(queries):
+            return app.response_class(
+                json_generator(query_func(queries), root='results'),
+                mimetype='application/json'
+            )
+        responder.__name__ = 'query_%s' % name
+        return responder
+    return make_responder
+
+
+def resource_list(name):
+    """Decorates a function to handle RESTful HTTP request for a list of
+    resources.
+    """
+    def make_responder(list_all):
+        def responder():
+            return app.response_class(
+                json_generator(list_all(), root=name),
+                mimetype='application/json'
+            )
+        responder.__name__ = 'all_%s' % name
+        return responder
+    return make_responder
+
+
+class IdListConverter(BaseConverter):
+    """Converts comma separated lists of ids in urls to integer lists.
+    """
+
+    def to_python(self, value):
+        ids = []
+        for id in value.split(','):
+            try:
+                ids.append(int(id))
+            except ValueError:
+                pass
+        return ids
+
+    def to_url(self, value):
+        return ','.join(value)
+
+
+class QueryConverter(PathConverter):
+    """Converts slash separated lists of queries in the url to string list.
+    """
+
+    def to_python(self, value):
+        return value.split('/')
+
+    def to_url(self, value):
+        return ','.join(value)
+
+
 # Flask setup.
 
 app = flask.Flask(__name__)
+app.url_map.converters['idlist'] = IdListConverter
+app.url_map.converters['query'] = QueryConverter
+
 
 @app.before_request
 def before_request():
@@ -61,17 +163,18 @@ def before_request():
 
 # Items.
 
-@app.route('/item/<int:item_id>')
-def single_item(item_id):
-    item = g.lib.get_item(item_id)
-    return flask.jsonify(_rep(item))
+@app.route('/item/<idlist:ids>')
+@resource('items')
+def get_item(id):
+    return g.lib.get_item(id)
+
 
 @app.route('/item/')
+@app.route('/item/query/')
+@resource_list('items')
 def all_items():
-    with g.lib.transaction() as tx:
-        rows = tx.query("SELECT id FROM items")
-    all_ids = [row[0] for row in rows]
-    return flask.jsonify(item_ids=all_ids)
+    return g.lib.items()
+
 
 @app.route('/item/<int:item_id>/file')
 def item_file(item_id):
@@ -81,32 +184,33 @@ def item_file(item_id):
     response.headers['Content-Length'] = os.path.getsize(item.path)
     return response
 
-@app.route('/item/query/<path:query>')
-def item_query(query):
-    parts = query.split('/')
-    items = g.lib.items(parts)
-    return flask.jsonify(results=[_rep(item) for item in items])
+
+@app.route('/item/query/<query:queries>')
+@resource_query('items')
+def item_query(queries):
+    return g.lib.items(queries)
 
 
 # Albums.
 
-@app.route('/album/<int:album_id>')
-def single_album(album_id):
-    album = g.lib.get_album(album_id)
-    return flask.jsonify(_rep(album))
+@app.route('/album/<idlist:ids>')
+@resource('albums')
+def get_album(id):
+    return g.lib.get_album(id)
+
 
 @app.route('/album/')
+@app.route('/album/query/')
+@resource_list('albums')
 def all_albums():
-    with g.lib.transaction() as tx:
-        rows = tx.query("SELECT id FROM albums")
-    all_ids = [row[0] for row in rows]
-    return flask.jsonify(album_ids=all_ids)
+    return g.lib.albums()
+
+
+@app.route('/album/query/<query:queries>')
+@resource_query('albums')
+def album_query(queries):
+    return g.lib.albums(queries)
 
-@app.route('/album/query/<path:query>')
-def album_query(query):
-    parts = query.split('/')
-    albums = g.lib.albums(parts)
-    return flask.jsonify(results=[_rep(album) for album in albums])
 
 @app.route('/album/<int:album_id>/art')
 def album_art(album_id):
@@ -158,6 +262,7 @@ class WebPlugin(BeetsPlugin):
         cmd = ui.Subcommand('web', help='start a Web interface')
         cmd.parser.add_option('-d', '--debug', action='store_true',
                               default=False, help='debug mode')
+
         def func(lib, opts, args):
             args = ui.decargs(args)
             if args:
