@@ -196,9 +196,10 @@ var ItemEntryView = Backbone.View.extend({
             this.$('.playing').hide();
     }
 });
-var ItemDetailView = Backbone.View.extend({
+//Holds Title, Artist, Album etc.
+var ItemMainDetailView = Backbone.View.extend({
     tagName: "div",
-    template: _.template($('#item-detail-template').html()),
+    template: _.template($('#item-main-detail-template').html()),
     events: {
         'click .play': 'play',
     },
@@ -210,7 +211,15 @@ var ItemDetailView = Backbone.View.extend({
         app.playItem(this.model);
     }
 });
-
+// Holds Track no., Format, MusicBrainz link, Lyrics, Comments etc.
+var ItemExtraDetailView = Backbone.View.extend({
+    tagName: "div",
+    template: _.template($('#item-extra-detail-template').html()),
+    render: function() {
+        $(this.el).html(this.template(this.model.toJSON()));
+        return this;
+    }
+});
 // Main app view.
 var AppView = Backbone.View.extend({
     el: $('body'),
@@ -246,9 +255,12 @@ var AppView = Backbone.View.extend({
         $('#results li').removeClass("selected");
         $(view.el).addClass("selected");
 
-        // Show detail.
-        var detailView = new ItemDetailView({model: view.model});
-        $('#detail').empty().append(detailView.render().el);
+        // Show main and extra detail.
+        var mainDetailView = new ItemMainDetailView({model: view.model});
+        $('#main-detail').empty().append(mainDetailView.render().el);
+
+        var extraDetailView = new ItemExtraDetailView({model: view.model});
+        $('#extra-detail').empty().append(extraDetailView.render().el);
     },
     playItem: function(item) {
         var url = '/item/' + item.get('id') + '/file';
