@@ -12,23 +12,24 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-""" Clears tag fields in media files.""" 
+""" Clears tag fields in media files."""
 
 import re
 import logging
 from beets.plugins import BeetsPlugin
-from beets.library import ITEM_KEYS
+from beets.library import Item
 from beets.importer import action
 from beets.util import confit
 
 __author__ = 'baobab@heresiarch.info'
 __version__ = '0.10'
 
+log = logging.getLogger('beets')
+
 
 class ZeroPlugin(BeetsPlugin):
 
     _instance = None
-    _log = logging.getLogger('beets')
 
     def __init__(self):
         super(ZeroPlugin, self).__init__()
@@ -45,25 +46,31 @@ class ZeroPlugin(BeetsPlugin):
         self.patterns = {}
         self.warned = False
 
-        for f in self.config['fields'].as_str_seq():
-            if f not in ITEM_KEYS:
-                self._log.error(u'[zero] invalid field: {0}'.format(f))
-            else:
-                try:
-                    self.patterns[f] = self.config[f].as_str_seq()
-                except confit.NotFoundError:
-                    self.patterns[f] = [u'']
+        for field in self.config['fields'].as_str_seq():
+            if field in ('id', 'path', 'album_id'):
+                log.warn(u'[zero] field \'{0}\' ignored, zeroing '
+                         u'it would be dangerous'.format(field))
+                continue
+            if field not in Item._fields.keys():
+                log.error(u'[zero] invalid field: {0}'.format(field))
+                continue
+
+            try:
+                self.patterns[field] = self.config[field].as_str_seq()
+            except confit.NotFoundError:
+                # Matches everything
+                self.patterns[field] = [u'']
 
     def import_task_choice_event(self, session, task):
         """Listen for import_task_choice event."""
         if task.choice_flag == action.ASIS and not self.warned:
-            self._log.warn(u'[zero] cannot zero in \"as-is\" mode')
+            log.warn(u'[zero] cannot zero in \"as-is\" mode')
             self.warned = True
-        # TODO request write in as-is mode 
+        # TODO request write in as-is mode
 
     @classmethod
     def match_patterns(cls, field, patterns):
-        """Check if field (as string) is matching any of the patterns in 
+        """Check if field (as string) is matching any of the patterns in
         the list.
         """
         for p in patterns:
@@ -71,25 +78,18 @@ class ZeroPlugin(BeetsPlugin):
                 return True
         return False
 
-    def write_event(self, item):
+    def write_event(self, item, path, tags):
         """Listen for write event."""
         if not self.patterns:
-            self._log.warn(u'[zero] no fields, nothing to do')
+            log.warn(u'[zero] no fields, nothing to do')
             return
-        for fn, patterns in self.patterns.items():
-            try:
-                fval = getattr(item, fn)
-            except AttributeError:
-                self._log.error(u'[zero] no such field: {0}'.format(fn))
-            else:
-                if not self.match_patterns(fval, patterns):
-                    self._log.debug(u'[zero] \"{0}\" ({1}) not match: {2}'
-                                    .format(fval, fn, 
-                                            ' '.join(patterns)))
-                    continue
-                self._log.debug(u'[zero] \"{0}\" ({1}) match: {2}'
-                                .format(fval, fn, ' '.join(patterns)))
-                new_val = None if fval is None else type(fval)()
-                setattr(item, fn, new_val)
-                self._log.debug(u'[zero] {0}={1}'
-                                .format(fn, getattr(item, fn)))
+
+        for field, patterns in self.patterns.items():
+            if field not in tags:
+                log.error(u'[zero] no such field: {0}'.format(field))
+                continue
+
+            value = tags[field]
+            if self.match_patterns(value, patterns):
+                log.debug(u'[zero] {0}: {1} -> None'.format(field, value))
+                tags[field] = None
