@@ -19,6 +19,7 @@ import os
 import logging
 import tempfile
 import shutil
+from contextlib import contextmanager
 
 # Use unittest2 on Python < 2.7.
 try:
@@ -33,56 +34,93 @@ from beets import importer
 from beets.ui import commands
 import beets
 
-# Suppress logging output.
-log = logging.getLogger('beets')
-log.setLevel(logging.CRITICAL)
+# Make sure the development versions of the plugins are used
+import beetsplug
+beetsplug.__path__ = [os.path.abspath(
+    os.path.join(__file__, '..', '..', 'beetsplug')
+)]
 
-# Test resources/sandbox path.
+# Test resources path.
 RSRC = os.path.join(os.path.dirname(__file__), 'rsrc')
 
+# Propagate to root loger so nosetest can capture it
+log = logging.getLogger('beets')
+log.propagate = True
+log.setLevel(logging.DEBUG)
+
 # Dummy item creation.
 _item_ident = 0
+
+
 def item(lib=None):
     global _item_ident
     _item_ident += 1
     i = beets.library.Item(
-        title =            u'the title',
-        artist =           u'the artist',
-        albumartist =      u'the album artist',
-        album =            u'the album',
-        genre =            u'the genre',
-        composer =         u'the composer',
-        grouping =         u'the grouping',
-        year =             1,
-        month =            2,
-        day =              3,
-        track =            4,
-        tracktotal =       5,
-        disc =             6,
-        disctotal =        7,
-        lyrics =           u'the lyrics',
-        comments =         u'the comments',
-        bpm =              8,
-        comp =             True,
-        path =             'somepath' + str(_item_ident),
-        length =           60.0,
-        bitrate =          128000,
-        format =           'FLAC',
-        mb_trackid =       'someID-1',
-        mb_albumid =       'someID-2',
-        mb_artistid =      'someID-3',
-        mb_albumartistid = 'someID-4',
-        album_id =         None,
+        title=u'the title',
+        artist=u'the artist',
+        albumartist=u'the album artist',
+        album=u'the album',
+        genre=u'the genre',
+        composer=u'the composer',
+        grouping=u'the grouping',
+        year=1,
+        month=2,
+        day=3,
+        track=4,
+        tracktotal=5,
+        disc=6,
+        disctotal=7,
+        lyrics=u'the lyrics',
+        comments=u'the comments',
+        bpm=8,
+        comp=True,
+        path='somepath' + str(_item_ident),
+        length=60.0,
+        bitrate=128000,
+        format='FLAC',
+        mb_trackid='someID-1',
+        mb_albumid='someID-2',
+        mb_artistid='someID-3',
+        mb_albumartistid='someID-4',
+        album_id=None,
     )
     if lib:
         lib.add(i)
     return i
 
+_album_ident = 0
+
+
+def album(lib=None):
+    global _item_ident
+    _item_ident += 1
+    i = beets.library.Album(
+        artpath=None,
+        albumartist='some album artist',
+        albumartist_sort='some sort album artist',
+        albumartist_credit='some album artist credit',
+        album='the album',
+        genre='the genre',
+        year=2014,
+        month=2,
+        day=5,
+        tracktotal=0,
+        disctotal=1,
+        comp=False,
+        mb_albumid='someID-1',
+        mb_albumartistid='someID-1'
+    )
+    if lib:
+        lib.add(i)
+    return i
+
+
 # Dummy import session.
 def import_session(lib=None, logfile=None, paths=[], query=[], cli=False):
     cls = commands.TerminalImportSession if cli else importer.ImportSession
     return cls(lib, logfile, paths, query)
 
+
 # A test harness for all beets tests.
 # Provides temporary, isolated configuration.
 class TestCase(unittest.TestCase):
@@ -115,16 +153,23 @@ class TestCase(unittest.TestCase):
     def tearDown(self):
         if os.path.isdir(self.temp_dir):
             shutil.rmtree(self.temp_dir)
-        os.environ['HOME'] = self._old_home
+        if self._old_home is None:
+            del os.environ['HOME']
+        else:
+            os.environ['HOME'] = self._old_home
         self.io.restore()
 
+        beets.config.clear()
+        beets.config._materialized = False
+
     def assertExists(self, path):
         self.assertTrue(os.path.exists(path),
                         'file does not exist: %s' % path)
 
     def assertNotExists(self, path):
         self.assertFalse(os.path.exists(path),
-                        'file exists: %s' % path)
+                         'file exists: %s' % path)
+
 
 class LibTestCase(TestCase):
     """A test case that includes an in-memory library object (`lib`) and
@@ -135,7 +180,9 @@ class LibTestCase(TestCase):
         self.lib = beets.library.Library(':memory:')
         self.i = item(self.lib)
 
-
+    def tearDown(self):
+        self.lib._connection().close()
+        super(LibTestCase, self).tearDown()
 
 
 # Mock timing.
@@ -171,29 +218,41 @@ class Timecop(object):
 class InputException(Exception):
     def __init__(self, output=None):
         self.output = output
+
     def __str__(self):
         msg = "Attempt to read with no input provided."
         if self.output is not None:
-            msg += " Output: %s" % repr(self.output)
+            msg += " Output: %s" % self.output
         return msg
+
+
 class DummyOut(object):
     encoding = 'utf8'
+
     def __init__(self):
         self.buf = []
+
     def write(self, s):
         self.buf.append(s)
+
     def get(self):
         return ''.join(self.buf)
+
     def clear(self):
         self.buf = []
+
+
 class DummyIn(object):
     encoding = 'utf8'
+
     def __init__(self, out=None):
         self.buf = []
         self.reads = 0
         self.out = out
+
     def add(self, s):
         self.buf.append(s + '\n')
+
     def readline(self):
         if not self.buf:
             if self.out:
@@ -202,6 +261,8 @@ class DummyIn(object):
                 raise InputException()
         self.reads += 1
         return self.buf.pop(0)
+
+
 class DummyIO(object):
     """Mocks input and output streams for testing UI code."""
     def __init__(self):
@@ -233,6 +294,7 @@ class DummyIO(object):
 def touch(path):
     open(path, 'a').close()
 
+
 class Bag(object):
     """An object that exposes a set of fields given as keyword
     arguments. Any field not found in the dictionary appears to be None.
@@ -243,3 +305,38 @@ class Bag(object):
 
     def __getattr__(self, key):
         return self.fields.get(key)
+
+
+# Platform mocking.
+
+@contextmanager
+def platform_windows():
+    import ntpath
+    old_path = os.path
+    try:
+        os.path = ntpath
+        yield
+    finally:
+        os.path = old_path
+
+
+@contextmanager
+def platform_posix():
+    import posixpath
+    old_path = os.path
+    try:
+        os.path = posixpath
+        yield
+    finally:
+        os.path = old_path
+
+
+@contextmanager
+def system_mock(name):
+    import platform
+    old_system = platform.system
+    platform.system = lambda: name
+    try:
+        yield
+    finally:
+        platform.system = old_system
