@@ -14,8 +14,6 @@
 
 """Tests for autotagging functionality.
 """
-import os
-import shutil
 import re
 import copy
 
@@ -29,6 +27,7 @@ from beets.util import plurality
 from beets.autotag import AlbumInfo, TrackInfo
 from beets import config
 
+
 class PluralityTest(_common.TestCase):
     def test_plurality_consensus(self):
         objs = [1, 1, 1, 1]
@@ -91,12 +90,14 @@ class PluralityTest(_common.TestCase):
         for f in fields:
             self.assertEqual(likelies[f], '%s_1' % f)
 
+
 def _make_item(title, track, artist=u'some artist'):
     return Item(title=title, track=track,
                 artist=artist, album=u'some album',
                 length=1,
                 mb_trackid='', mb_albumid='', mb_artistid='')
 
+
 def _make_trackinfo():
     return [
         TrackInfo(u'one', None, u'some artist', length=1, index=1),
@@ -104,7 +105,19 @@ def _make_trackinfo():
         TrackInfo(u'three', None, u'some artist', length=1, index=3),
     ]
 
+
+def _clear_weights():
+    """Hack around the lazy descriptor used to cache weights for
+    Distance calculations.
+    """
+    Distance.__dict__['_weights'].computed = False
+
+
 class DistanceTest(_common.TestCase):
+    def tearDown(self):
+        super(DistanceTest, self).tearDown()
+        _clear_weights()
+
     def test_add(self):
         dist = Distance()
         dist.add('add', 1.0)
@@ -144,7 +157,7 @@ class DistanceTest(_common.TestCase):
 
         dist.add_number('number', -1, 2)
         self.assertEqual(dist._penalties['number'], [0.0, 1.0, 1.0, 1.0,
-                                                          1.0, 1.0])
+                                                     1.0, 1.0])
 
     def test_add_priority(self):
         dist = Distance()
@@ -155,12 +168,12 @@ class DistanceTest(_common.TestCase):
         self.assertEqual(dist._penalties['priority'], [0.0, 0.5])
 
         dist.add_priority('priority', 'gh', ['ab', 'cd', 'ef',
-                                                  re.compile('GH', re.I)])
+                                             re.compile('GH', re.I)])
         self.assertEqual(dist._penalties['priority'], [0.0, 0.5, 0.75])
 
         dist.add_priority('priority', 'xyz', ['abc', 'def'])
         self.assertEqual(dist._penalties['priority'], [0.0, 0.5, 0.75,
-                                                            1.0])
+                                                       1.0])
 
     def test_add_ratio(self):
         dist = Distance()
@@ -181,10 +194,23 @@ class DistanceTest(_common.TestCase):
         sdist = string_dist(u'abc', u'bcd')
         dist.add_string('string', u'abc', u'bcd')
         self.assertEqual(dist._penalties['string'], [sdist])
+        self.assertNotEqual(dist._penalties['string'], [0])
+
+    def test_add_string_none(self):
+        dist = Distance()
+        dist.add_string('string', None, 'string')
+        self.assertEqual(dist._penalties['string'], [1])
+
+    def test_add_string_both_none(self):
+        dist = Distance()
+        dist.add_string('string', None, None)
+        self.assertEqual(dist._penalties['string'], [0])
 
     def test_distance(self):
         config['match']['distance_weights']['album'] = 2.0
         config['match']['distance_weights']['medium'] = 1.0
+        _clear_weights()
+
         dist = Distance()
         dist.add('album', 0.5)
         dist.add('media', 0.25)
@@ -198,6 +224,8 @@ class DistanceTest(_common.TestCase):
     def test_max_distance(self):
         config['match']['distance_weights']['album'] = 3.0
         config['match']['distance_weights']['medium'] = 1.0
+        _clear_weights()
+
         dist = Distance()
         dist.add('album', 0.5)
         dist.add('medium', 0.0)
@@ -208,6 +236,8 @@ class DistanceTest(_common.TestCase):
         config['match']['distance_weights']['source'] = 1.0
         config['match']['distance_weights']['album'] = 2.0
         config['match']['distance_weights']['medium'] = 1.0
+        _clear_weights()
+
         dist = Distance()
         dist.add('source', 0.0)
         dist.add('album', 0.5)
@@ -225,6 +255,8 @@ class DistanceTest(_common.TestCase):
     def test_raw_distance(self):
         config['match']['distance_weights']['album'] = 3.0
         config['match']['distance_weights']['medium'] = 1.0
+        _clear_weights()
+
         dist = Distance()
         dist.add('album', 0.5)
         dist.add('medium', 0.25)
@@ -234,6 +266,8 @@ class DistanceTest(_common.TestCase):
     def test_items(self):
         config['match']['distance_weights']['album'] = 4.0
         config['match']['distance_weights']['medium'] = 2.0
+        _clear_weights()
+
         dist = Distance()
         dist.add('album', 0.1875)
         dist.add('medium', 0.75)
@@ -258,7 +292,8 @@ class DistanceTest(_common.TestCase):
         dist1.update(dist2)
 
         self.assertEqual(dist1._penalties, {'album': [0.5, 0.75, 0.25],
-                                             'media': [1.0, 0.05]})
+                                            'media': [1.0, 0.05]})
+
 
 class TrackDistanceTest(_common.TestCase):
     def test_identical_tracks(self):
@@ -287,6 +322,7 @@ class TrackDistanceTest(_common.TestCase):
         dist = match.track_distance(item, info, incl_artist=True)
         self.assertEqual(dist, 0.0)
 
+
 class AlbumDistanceTest(_common.TestCase):
     def _mapping(self, items, info):
         out = {}
@@ -303,11 +339,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         self.assertEqual(self._dist(items, info), 0)
 
@@ -316,11 +353,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'one', 1))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         dist = self._dist(items, info)
         self.assertNotEqual(dist, 0)
@@ -333,11 +371,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'someone else',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'someone else',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         self.assertNotEqual(self._dist(items, info), 0)
 
@@ -347,11 +386,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'should be ignored',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = True,
-            album_id = None, artist_id = None,
+            artist=u'should be ignored',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=True,
+            album_id=None,
+            artist_id=None,
         )
         self.assertEqual(self._dist(items, info), 0)
 
@@ -362,11 +402,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'should be ignored',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = True,
-            album_id = None, artist_id = None,
+            artist=u'should be ignored',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=True,
+            album_id=None,
+            artist_id=None,
         )
         info.tracks[0].artist = None
         info.tracks[1].artist = None
@@ -379,11 +420,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2, u'someone else'))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = True,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=True,
+            album_id=None,
+            artist_id=None,
         )
         self.assertNotEqual(self._dist(items, info), 0)
 
@@ -393,11 +435,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'three', 2))
         items.append(_make_item(u'two', 3))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         dist = self._dist(items, info)
         self.assertTrue(0 < dist < 0.2)
@@ -408,11 +451,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 3))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         info.tracks[0].medium_index = 1
         info.tracks[1].medium_index = 2
@@ -426,11 +470,12 @@ class AlbumDistanceTest(_common.TestCase):
         items.append(_make_item(u'two', 2))
         items.append(_make_item(u'three', 1))
         info = AlbumInfo(
-            artist = u'some artist',
-            album = u'some album',
-            tracks = _make_trackinfo(),
-            va = False,
-            album_id = None, artist_id = None,
+            artist=u'some artist',
+            album=u'some album',
+            tracks=_make_trackinfo(),
+            va=False,
+            album_id=None,
+            artist_id=None,
         )
         info.tracks[0].medium_index = 1
         info.tracks[1].medium_index = 2
@@ -438,123 +483,6 @@ class AlbumDistanceTest(_common.TestCase):
         dist = self._dist(items, info)
         self.assertEqual(dist, 0)
 
-def _mkmp3(path):
-    shutil.copyfile(os.path.join(_common.RSRC, 'min.mp3'), path)
-class AlbumsInDirTest(_common.TestCase):
-    def setUp(self):
-        super(AlbumsInDirTest, self).setUp()
-
-        # create a directory structure for testing
-        self.base = os.path.abspath(os.path.join(self.temp_dir, 'tempdir'))
-        os.mkdir(self.base)
-
-        os.mkdir(os.path.join(self.base, 'album1'))
-        os.mkdir(os.path.join(self.base, 'album2'))
-        os.mkdir(os.path.join(self.base, 'more'))
-        os.mkdir(os.path.join(self.base, 'more', 'album3'))
-        os.mkdir(os.path.join(self.base, 'more', 'album4'))
-
-        _mkmp3(os.path.join(self.base, 'album1', 'album1song1.mp3'))
-        _mkmp3(os.path.join(self.base, 'album1', 'album1song2.mp3'))
-        _mkmp3(os.path.join(self.base, 'album2', 'album2song.mp3'))
-        _mkmp3(os.path.join(self.base, 'more', 'album3', 'album3song.mp3'))
-        _mkmp3(os.path.join(self.base, 'more', 'album4', 'album4song.mp3'))
-
-    def test_finds_all_albums(self):
-        albums = list(autotag.albums_in_dir(self.base))
-        self.assertEqual(len(albums), 4)
-
-    def test_separates_contents(self):
-        found = []
-        for _, album in autotag.albums_in_dir(self.base):
-            found.append(re.search(r'album(.)song', album[0].path).group(1))
-        self.assertTrue('1' in found)
-        self.assertTrue('2' in found)
-        self.assertTrue('3' in found)
-        self.assertTrue('4' in found)
-
-    def test_finds_multiple_songs(self):
-        for _, album in autotag.albums_in_dir(self.base):
-            n = re.search(r'album(.)song', album[0].path).group(1)
-            if n == '1':
-                self.assertEqual(len(album), 2)
-            else:
-                self.assertEqual(len(album), 1)
-
-class MultiDiscAlbumsInDirTest(_common.TestCase):
-    def setUp(self):
-        super(MultiDiscAlbumsInDirTest, self).setUp()
-
-        self.base = os.path.abspath(os.path.join(self.temp_dir, 'tempdir'))
-        os.mkdir(self.base)
-
-        self.dirs = [
-            # Nested album, multiple subdirs.
-            # Also, false positive marker in root dir, and subtitle for disc 3.
-            os.path.join(self.base, 'ABCD1234'),
-            os.path.join(self.base, 'ABCD1234', 'cd 1'),
-            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus'),
-
-            # Nested album, single subdir.
-            # Also, punctuation between marker and disc number.
-            os.path.join(self.base, 'album'),
-            os.path.join(self.base, 'album', 'cd _ 1'),
-
-            # Flattened album, case typo.
-            # Also, false positive marker in parent dir.
-            os.path.join(self.base, 'artist [CD5]'),
-            os.path.join(self.base, 'artist [CD5]', 'CAT disc 1'),
-            os.path.join(self.base, 'artist [CD5]', 'CAt disc 2'),
-
-            # Single disc album, sorted between CAT discs.
-            os.path.join(self.base, 'artist [CD5]', 'CATS'),
-        ]
-        self.files = [
-            os.path.join(self.base, 'ABCD1234', 'cd 1', 'song1.mp3'),
-            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus', 'song2.mp3'),
-            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus', 'song3.mp3'),
-            os.path.join(self.base, 'album', 'cd _ 1', 'song4.mp3'),
-            os.path.join(self.base, 'artist [CD5]', 'CAT disc 1', 'song5.mp3'),
-            os.path.join(self.base, 'artist [CD5]', 'CAt disc 2', 'song6.mp3'),
-            os.path.join(self.base, 'artist [CD5]', 'CATS', 'song7.mp3'),
-        ]
-
-        for path in self.dirs:
-            os.mkdir(path)
-        for path in self.files:
-            _mkmp3(path)
-
-    def test_coalesce_nested_album_multiple_subdirs(self):
-        albums = list(autotag.albums_in_dir(self.base))
-        self.assertEquals(len(albums), 4)
-        root, items = albums[0]
-        self.assertEquals(root, self.dirs[0:3])
-        self.assertEquals(len(items), 3)
-
-    def test_coalesce_nested_album_single_subdir(self):
-        albums = list(autotag.albums_in_dir(self.base))
-        root, items = albums[1]
-        self.assertEquals(root, self.dirs[3:5])
-        self.assertEquals(len(items), 1)
-
-    def test_coalesce_flattened_album_case_typo(self):
-        albums = list(autotag.albums_in_dir(self.base))
-        root, items = albums[2]
-        self.assertEquals(root, self.dirs[6:8])
-        self.assertEquals(len(items), 2)
-
-    def test_single_disc_album(self):
-        albums = list(autotag.albums_in_dir(self.base))
-        root, items = albums[3]
-        self.assertEquals(root, self.dirs[8:])
-        self.assertEquals(len(items), 1)
-
-    def test_do_not_yield_empty_album(self):
-        # Remove all the MP3s.
-        for path in self.files:
-            os.remove(path)
-        albums = list(autotag.albums_in_dir(self.base))
-        self.assertEquals(len(albums), 0)
 
 class AssignmentTest(unittest.TestCase):
     def item(self, title, track):
@@ -671,7 +599,7 @@ class AssignmentTest(unittest.TestCase):
         trackinfo.append(info(6, u'Suzie Blue', 269.30599999999998))
         trackinfo.append(info(7, u'Steal My Kisses', 245.36000000000001))
         trackinfo.append(info(8, u'Burn to Shine', 214.90600000000001))
-        trackinfo.append(info(9, u'Show Me a Little Shame', 224.09299999999999))
+        trackinfo.append(info(9, u'Show Me a Little Shame', 224.0929999999999))
         trackinfo.append(info(10, u'Forgiven', 317.19999999999999))
         trackinfo.append(info(11, u'Beloved One', 243.733))
         trackinfo.append(info(12, u'In the Lord\'s Arms', 186.13300000000001))
@@ -683,6 +611,7 @@ class AssignmentTest(unittest.TestCase):
         for item, info in mapping.iteritems():
             self.assertEqual(items.index(item), trackinfo.index(info))
 
+
 class ApplyTestUtil(object):
     def _apply(self, info=None, per_disc_numbering=False):
         info = info or self.info
@@ -692,6 +621,7 @@ class ApplyTestUtil(object):
         config['per_disc_numbering'] = per_disc_numbering
         autotag.apply_metadata(info, mapping)
 
+
 class ApplyTest(_common.TestCase, ApplyTestUtil):
     def setUp(self):
         super(ApplyTest, self).setUp()
@@ -719,16 +649,16 @@ class ApplyTest(_common.TestCase, ApplyTestUtil):
             medium_total=1,
         ))
         self.info = AlbumInfo(
-            tracks = trackinfo,
-            artist = u'artistNew',
-            album = u'albumNew',
-            album_id = '7edb51cb-77d6-4416-a23c-3a8c2994a2c7',
-            artist_id = 'a6623d39-2d8e-4f70-8242-0a9553b91e50',
-            artist_credit = u'albumArtistCredit',
-            artist_sort = u'albumArtistSort',
-            albumtype = u'album',
-            va = False,
-            mediums = 2,
+            tracks=trackinfo,
+            artist=u'artistNew',
+            album=u'albumNew',
+            album_id='7edb51cb-77d6-4416-a23c-3a8c2994a2c7',
+            artist_id='a6623d39-2d8e-4f70-8242-0a9553b91e50',
+            artist_credit=u'albumArtistCredit',
+            artist_sort=u'albumArtistSort',
+            albumtype=u'album',
+            va=False,
+            mediums=2,
         )
 
     def test_titles_applied(self):
@@ -776,7 +706,7 @@ class ApplyTest(_common.TestCase, ApplyTestUtil):
     def test_mb_trackid_applied(self):
         self._apply()
         self.assertEqual(self.items[0].mb_trackid,
-                        'dfa939ec-118c-4d0f-84a0-60f3d1e6522c')
+                         'dfa939ec-118c-4d0f-84a0-60f3d1e6522c')
         self.assertEqual(self.items[1].mb_trackid,
                          '40130ed1-a27c-42fd-a328-1ebefb6caef4')
 
@@ -821,6 +751,42 @@ class ApplyTest(_common.TestCase, ApplyTestUtil):
         self.assertEqual(self.items[1].albumartist_sort, 'albumArtistSort')
         self.assertEqual(self.items[1].artist_sort, 'albumArtistSort')
 
+    def test_full_date_applied(self):
+        my_info = copy.deepcopy(self.info)
+        my_info.year = 2013
+        my_info.month = 12
+        my_info.day = 18
+        self._apply(info=my_info)
+
+        self.assertEqual(self.items[0].year, 2013)
+        self.assertEqual(self.items[0].month, 12)
+        self.assertEqual(self.items[0].day, 18)
+
+    def test_date_only_zeros_month_and_day(self):
+        self.items = []
+        self.items.append(Item(year=1, month=2, day=3))
+        self.items.append(Item(year=4, month=5, day=6))
+
+        my_info = copy.deepcopy(self.info)
+        my_info.year = 2013
+        self._apply(info=my_info)
+
+        self.assertEqual(self.items[0].year, 2013)
+        self.assertEqual(self.items[0].month, 0)
+        self.assertEqual(self.items[0].day, 0)
+
+    def test_missing_date_applies_nothing(self):
+        self.items = []
+        self.items.append(Item(year=1, month=2, day=3))
+        self.items.append(Item(year=4, month=5, day=6))
+
+        self._apply()
+
+        self.assertEqual(self.items[0].year, 1)
+        self.assertEqual(self.items[0].month, 2)
+        self.assertEqual(self.items[0].day, 3)
+
+
 class ApplyCompilationTest(_common.TestCase, ApplyTestUtil):
     def setUp(self):
         super(ApplyCompilationTest, self).setUp()
@@ -844,13 +810,12 @@ class ApplyCompilationTest(_common.TestCase, ApplyTestUtil):
             index=2,
         ))
         self.info = AlbumInfo(
-            tracks = trackinfo,
-            artist = u'variousNew',
-            album = u'albumNew',
-            album_id = '3b69ea40-39b8-487f-8818-04b6eff8c21a',
-            artist_id = '89ad4ac3-39f7-470e-963a-56509c546377',
-            albumtype = u'compilation',
-            va = False,
+            tracks=trackinfo,
+            artist=u'variousNew',
+            album=u'albumNew',
+            album_id='3b69ea40-39b8-487f-8818-04b6eff8c21a',
+            artist_id='89ad4ac3-39f7-470e-963a-56509c546377',
+            albumtype=u'compilation',
         )
 
     def test_album_and_track_artists_separate(self):
@@ -883,6 +848,7 @@ class ApplyCompilationTest(_common.TestCase, ApplyTestUtil):
         self.assertTrue(self.items[0].comp)
         self.assertTrue(self.items[1].comp)
 
+
 class StringDistanceTest(unittest.TestCase):
     def test_equal_strings(self):
         dist = string_dist(u'Some String', u'Some String')
@@ -958,6 +924,21 @@ class StringDistanceTest(unittest.TestCase):
         dist = string_dist(u'\xe9\xe1\xf1', u'ean')
         self.assertEqual(dist, 0.0)
 
+
+class EnumTest(_common.TestCase):
+    """
+    Test Enum Subclasses defined in beets.util.enumeration
+    """
+    def test_ordered_enum(self):
+        OrderedEnumTest = match.OrderedEnum('OrderedEnumTest', ['a', 'b', 'c'])
+        self.assertLess(OrderedEnumTest.a, OrderedEnumTest.b)
+        self.assertLess(OrderedEnumTest.a, OrderedEnumTest.c)
+        self.assertLess(OrderedEnumTest.b, OrderedEnumTest.c)
+        self.assertGreater(OrderedEnumTest.b, OrderedEnumTest.a)
+        self.assertGreater(OrderedEnumTest.c, OrderedEnumTest.a)
+        self.assertGreater(OrderedEnumTest.c, OrderedEnumTest.b)
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
