@@ -19,6 +19,7 @@ from _common import unittest
 from beets.autotag import mb
 from beets import config
 
+
 class MBAlbumInfoTest(_common.TestCase):
     def _make_release(self, date_str='2009', tracks=None, track_length=None,
                       track_artist=False):
@@ -311,6 +312,7 @@ class MBAlbumInfoTest(_common.TestCase):
         self.assertEqual(track.artist_sort, 'TRACK ARTIST SORT NAME')
         self.assertEqual(track.artist_credit, 'TRACK ARTIST CREDIT')
 
+
 class ParseIDTest(_common.TestCase):
     def test_parse_id_correct(self):
         id_string = "28e32c71-1450-463e-92bf-e0a46446fc11"
@@ -328,6 +330,7 @@ class ParseIDTest(_common.TestCase):
         out = mb._parse_id(id_url)
         self.assertEqual(out, id_string)
 
+
 class ArtistFlatteningTest(_common.TestCase):
     def _credit_dict(self, suffix=''):
         return {
@@ -366,27 +369,29 @@ class ArtistFlatteningTest(_common.TestCase):
 
     def test_alias(self):
         credit_dict = self._credit_dict()
-        self._add_alias(credit_dict, suffix='en', locale='en')
-        self._add_alias(credit_dict, suffix='en_GB', locale='en_GB')
+        self._add_alias(credit_dict, suffix='en', locale='en', primary=True)
+        self._add_alias(credit_dict, suffix='en_GB', locale='en_GB',
+                        primary=True)
         self._add_alias(credit_dict, suffix='fr', locale='fr')
         self._add_alias(credit_dict, suffix='fr_P', locale='fr', primary=True)
+        self._add_alias(credit_dict, suffix='pt_BR', locale='pt_BR')
 
         # test no alias
         config['import']['languages'] = ['']
         flat = mb._flatten_artist_credit([credit_dict])
         self.assertEqual(flat, ('NAME', 'SORT', 'CREDIT'))
 
-        # test en
+        # test en primary
         config['import']['languages'] = ['en']
         flat = mb._flatten_artist_credit([credit_dict])
         self.assertEqual(flat, ('ALIASen', 'ALIASSORTen', 'CREDIT'))
 
-        # test en_GB en
+        # test en_GB en primary
         config['import']['languages'] = ['en_GB', 'en']
         flat = mb._flatten_artist_credit([credit_dict])
         self.assertEqual(flat, ('ALIASen_GB', 'ALIASSORTen_GB', 'CREDIT'))
 
-        # test en en_GB
+        # test en en_GB primary
         config['import']['languages'] = ['en', 'en_GB']
         flat = mb._flatten_artist_credit([credit_dict])
         self.assertEqual(flat, ('ALIASen', 'ALIASSORTen', 'CREDIT'))
@@ -396,6 +401,12 @@ class ArtistFlatteningTest(_common.TestCase):
         flat = mb._flatten_artist_credit([credit_dict])
         self.assertEqual(flat, ('ALIASfr_P', 'ALIASSORTfr_P', 'CREDIT'))
 
+        # test for not matching non-primary
+        config['import']['languages'] = ['pt_BR', 'fr']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('ALIASfr_P', 'ALIASSORTfr_P', 'CREDIT'))
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
