@@ -16,72 +16,42 @@
 """
 import _common
 from _common import unittest
+import helper
+
 import beets.library
-
-pqp = beets.library.parse_query_part
+from beets import dbcore
+from beets.dbcore import types
+from beets.dbcore.query import NoneQuery
+from beets.library import Library, Item
 
 
-class QueryParseTest(_common.TestCase):
-    def test_one_basic_term(self):
-        q = 'test'
-        r = (None, 'test', beets.library.SubstringQuery)
-        self.assertEqual(pqp(q), r)
+class TestHelper(helper.TestHelper):
 
-    def test_one_keyed_term(self):
-        q = 'test:val'
-        r = ('test', 'val', beets.library.SubstringQuery)
-        self.assertEqual(pqp(q), r)
+    def assertInResult(self, item, results):
+        result_ids = map(lambda i: i.id, results)
+        self.assertIn(item.id, result_ids)
 
-    def test_colon_at_end(self):
-        q = 'test:'
-        r = (None, 'test:', beets.library.SubstringQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_one_basic_regexp(self):
-        q = r':regexp'
-        r = (None, 'regexp', beets.library.RegexpQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_keyed_regexp(self):
-        q = r'test::regexp'
-        r = ('test', 'regexp', beets.library.RegexpQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_escaped_colon(self):
-        q = r'test\:val'
-        r = (None, 'test:val', beets.library.SubstringQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_escaped_colon_in_regexp(self):
-        q = r':test\:regexp'
-        r = (None, 'test:regexp', beets.library.RegexpQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_single_year(self):
-        q = 'year:1999'
-        r = ('year', '1999', beets.library.NumericQuery)
-        self.assertEqual(pqp(q), r)
-
-    def test_multiple_years(self):
-        q = 'year:1999..2010'
-        r = ('year', '1999..2010', beets.library.NumericQuery)
-        self.assertEqual(pqp(q), r)
+    def assertNotInResult(self, item, results):
+        result_ids = map(lambda i: i.id, results)
+        self.assertNotIn(item.id, result_ids)
 
 
 class AnyFieldQueryTest(_common.LibTestCase):
     def test_no_restriction(self):
-        q = beets.library.AnyFieldQuery('title', beets.library.ITEM_KEYS,
-                                        beets.library.SubstringQuery)
+        q = dbcore.query.AnyFieldQuery(
+            'title', beets.library.Item._fields.keys(),
+            dbcore.query.SubstringQuery
+        )
         self.assertEqual(self.lib.items(q).get().title, 'the title')
 
     def test_restriction_completeness(self):
-        q = beets.library.AnyFieldQuery('title', ['title'],
-                                        beets.library.SubstringQuery)
+        q = dbcore.query.AnyFieldQuery('title', ['title'],
+                                       dbcore.query.SubstringQuery)
         self.assertEqual(self.lib.items(q).get().title, 'the title')
 
     def test_restriction_soundness(self):
-        q = beets.library.AnyFieldQuery('title', ['artist'],
-                                        beets.library.SubstringQuery)
+        q = dbcore.query.AnyFieldQuery('title', ['artist'],
+                                       dbcore.query.SubstringQuery)
         self.assertEqual(self.lib.items(q).get(), None)
 
 
@@ -250,7 +220,8 @@ class GetTest(DummyDataTestCase):
 
     def test_bad_year(self):
         q = 'year:delete from items'
-        self.assertRaises(ValueError, self.lib.items, q)
+        results = self.lib.items(q)
+        self.assert_matched(results, [])
 
     def test_singleton_true(self):
         q = 'singleton:true'
@@ -300,15 +271,20 @@ class GetTest(DummyDataTestCase):
         self.assert_matched(results, [u'caf\xe9'])
 
     def test_numeric_search_positive(self):
-        q = beets.library.NumericQuery('year', '2001')
+        q = dbcore.query.NumericQuery('year', '2001')
         results = self.lib.items(q)
         self.assertTrue(results)
 
     def test_numeric_search_negative(self):
-        q = beets.library.NumericQuery('year', '1999')
+        q = dbcore.query.NumericQuery('year', '1999')
         results = self.lib.items(q)
         self.assertFalse(results)
 
+    def test_numeric_empty(self):
+        q = dbcore.query.NumericQuery('year', '')
+        results = self.lib.items(q)
+        self.assertTrue(results)
+
 
 class MatchTest(_common.TestCase):
     def setUp(self):
@@ -316,43 +292,43 @@ class MatchTest(_common.TestCase):
         self.item = _common.item()
 
     def test_regex_match_positive(self):
-        q = beets.library.RegexpQuery('album', '^the album$')
+        q = dbcore.query.RegexpQuery('album', '^the album$')
         self.assertTrue(q.match(self.item))
 
     def test_regex_match_negative(self):
-        q = beets.library.RegexpQuery('album', '^album$')
+        q = dbcore.query.RegexpQuery('album', '^album$')
         self.assertFalse(q.match(self.item))
 
     def test_regex_match_non_string_value(self):
-        q = beets.library.RegexpQuery('disc', '^6$')
+        q = dbcore.query.RegexpQuery('disc', '^6$')
         self.assertTrue(q.match(self.item))
 
     def test_substring_match_positive(self):
-        q = beets.library.SubstringQuery('album', 'album')
+        q = dbcore.query.SubstringQuery('album', 'album')
         self.assertTrue(q.match(self.item))
 
     def test_substring_match_negative(self):
-        q = beets.library.SubstringQuery('album', 'ablum')
+        q = dbcore.query.SubstringQuery('album', 'ablum')
         self.assertFalse(q.match(self.item))
 
     def test_substring_match_non_string_value(self):
-        q = beets.library.SubstringQuery('disc', '6')
+        q = dbcore.query.SubstringQuery('disc', '6')
         self.assertTrue(q.match(self.item))
 
     def test_year_match_positive(self):
-        q = beets.library.NumericQuery('year', '1')
+        q = dbcore.query.NumericQuery('year', '1')
         self.assertTrue(q.match(self.item))
 
     def test_year_match_negative(self):
-        q = beets.library.NumericQuery('year', '10')
+        q = dbcore.query.NumericQuery('year', '10')
         self.assertFalse(q.match(self.item))
 
     def test_bitrate_range_positive(self):
-        q = beets.library.NumericQuery('bitrate', '100000..200000')
+        q = dbcore.query.NumericQuery('bitrate', '100000..200000')
         self.assertTrue(q.match(self.item))
 
     def test_bitrate_range_negative(self):
-        q = beets.library.NumericQuery('bitrate', '200000..300000')
+        q = dbcore.query.NumericQuery('bitrate', '200000..300000')
         self.assertFalse(q.match(self.item))
 
 
@@ -414,6 +390,98 @@ class PathQueryTest(_common.LibTestCase, AssertsMixin):
         self.assert_matched(results, ['path item'])
 
 
+class IntQueryTest(unittest.TestCase, TestHelper):
+
+    def setUp(self):
+        self.lib = Library(':memory:')
+
+    def tearDown(self):
+        Item._types = {}
+
+    def test_exact_value_match(self):
+        item = self.add_item(bpm=120)
+        matched = self.lib.items('bpm:120').get()
+        self.assertEqual(item.id, matched.id)
+
+    def test_range_match(self):
+        item = self.add_item(bpm=120)
+        self.add_item(bpm=130)
+
+        matched = self.lib.items('bpm:110..125')
+        self.assertEqual(1, len(matched))
+        self.assertEqual(item.id, matched.get().id)
+
+    def test_flex_range_match(self):
+        Item._types = {'myint': types.Integer()}
+        item = self.add_item(myint=2)
+        matched = self.lib.items('myint:2').get()
+        self.assertEqual(item.id, matched.id)
+
+    def test_flex_dont_match_missing(self):
+        Item._types = {'myint': types.Integer()}
+        self.add_item()
+        matched = self.lib.items('myint:2').get()
+        self.assertIsNone(matched)
+
+    def test_no_substring_match(self):
+        self.add_item(bpm=120)
+        matched = self.lib.items('bpm:12').get()
+        self.assertIsNone(matched)
+
+
+class BoolQueryTest(unittest.TestCase, TestHelper):
+
+    def setUp(self):
+        self.lib = Library(':memory:')
+        Item._types = {'flexbool': types.Boolean()}
+
+    def tearDown(self):
+        Item._types = {}
+
+    def test_parse_true(self):
+        item_true = self.add_item(comp=True)
+        item_false = self.add_item(comp=False)
+        matched = self.lib.items('comp:true')
+        self.assertInResult(item_true, matched)
+        self.assertNotInResult(item_false, matched)
+
+    def test_flex_parse_true(self):
+        item_true = self.add_item(flexbool=True)
+        item_false = self.add_item(flexbool=False)
+        matched = self.lib.items('flexbool:true')
+        self.assertInResult(item_true, matched)
+        self.assertNotInResult(item_false, matched)
+
+    def test_flex_parse_false(self):
+        item_true = self.add_item(flexbool=True)
+        item_false = self.add_item(flexbool=False)
+        matched = self.lib.items('flexbool:false')
+        self.assertInResult(item_false, matched)
+        self.assertNotInResult(item_true, matched)
+
+    def test_flex_parse_1(self):
+        item_true = self.add_item(flexbool=True)
+        item_false = self.add_item(flexbool=False)
+        matched = self.lib.items('flexbool:1')
+        self.assertInResult(item_true, matched)
+        self.assertNotInResult(item_false, matched)
+
+    def test_flex_parse_0(self):
+        item_true = self.add_item(flexbool=True)
+        item_false = self.add_item(flexbool=False)
+        matched = self.lib.items('flexbool:0')
+        self.assertInResult(item_false, matched)
+        self.assertNotInResult(item_true, matched)
+
+    def test_flex_parse_any_string(self):
+        # TODO this should be the other way around
+        item_true = self.add_item(flexbool=True)
+        item_false = self.add_item(flexbool=False)
+        matched = self.lib.items('flexbool:something')
+        self.assertInResult(item_false, matched)
+        self.assertNotInResult(item_true, matched)
+
+
 class DefaultSearchFieldsTest(DummyDataTestCase):
     def test_albums_matches_album(self):
         albums = list(self.lib.albums('baz'))
@@ -432,14 +500,28 @@ class DefaultSearchFieldsTest(DummyDataTestCase):
         self.assert_matched(items, [])
 
 
-class StringParseTest(_common.TestCase):
-    def test_single_field_query(self):
-        q = beets.library.AndQuery.from_string(u'albumtype:soundtrack')
-        self.assertEqual(len(q.subqueries), 1)
-        subq = q.subqueries[0]
-        self.assertTrue(isinstance(subq, beets.library.SubstringQuery))
-        self.assertEqual(subq.field, 'albumtype')
-        self.assertEqual(subq.pattern, 'soundtrack')
+class NoneQueryTest(unittest.TestCase, TestHelper):
+
+    def setUp(self):
+        self.lib = Library(':memory:')
+
+    def test_match_singletons(self):
+        singleton = self.add_item()
+        album_item = self.add_album().items().get()
+
+        matched = self.lib.items(NoneQuery('album_id'))
+        self.assertInResult(singleton, matched)
+        self.assertNotInResult(album_item, matched)
+
+    def test_match_after_set_none(self):
+        item = self.add_item(rg_track_gain=0)
+        matched = self.lib.items(NoneQuery('rg_track_gain'))
+        self.assertNotInResult(item, matched)
+
+        item['rg_track_gain'] = None
+        item.store()
+        matched = self.lib.items(NoneQuery('rg_track_gain'))
+        self.assertInResult(item, matched)
 
 
 def suite():
