@@ -17,6 +17,7 @@
 from _common import unittest
 from beets.util import functemplate
 
+
 def _normexpr(expr):
     """Normalize an Expression object's parts, collapsing multiple
     adjacent text blocks and removing empty text blocks. Generates a
@@ -38,10 +39,12 @@ def _normexpr(expr):
         if text:
             yield text
 
+
 def _normparse(text):
     """Parse a template and then normalize the resulting Expression."""
     return _normexpr(functemplate._parse(text))
 
+
 class ParseTest(unittest.TestCase):
     def test_empty_string(self):
         self.assertEqual(list(_normparse(u'')), [])
@@ -139,7 +142,8 @@ class ParseTest(unittest.TestCase):
         self.assertEqual(list(_normparse(u'foo %bar baz')), [u'foo %bar baz'])
 
     def test_call_with_unclosed_args(self):
-        self.assertEqual(list(_normparse(u'foo %bar{ baz')), [u'foo %bar{ baz'])
+        self.assertEqual(list(_normparse(u'foo %bar{ baz')),
+                         [u'foo %bar{ baz'])
 
     def test_call_with_unclosed_multiple_args(self):
         self.assertEqual(list(_normparse(u'foo %bar{bar,bar baz')),
@@ -203,6 +207,7 @@ class ParseTest(unittest.TestCase):
         self._assert_call(arg_parts[0], u"bar", 1)
         self.assertEqual(list(_normexpr(arg_parts[0].args[0])), [u'baz'])
 
+
 class EvalTest(unittest.TestCase):
     def _eval(self, template):
         values = {
@@ -255,6 +260,7 @@ class EvalTest(unittest.TestCase):
     def test_function_call_with_empty_arg(self):
         self.assertEqual(self._eval(u"%len{}"), u"0")
 
+
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
